/**
 * 
 */
package com.github.ansell.oas.test;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.TreeSet;
import java.util.UUID;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.openrdf.model.Statement;
import org.openrdf.model.ValueFactory;
import org.openrdf.model.impl.ValueFactoryImpl;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.RDFParser;
import org.openrdf.rio.Rio;
import org.openrdf.rio.helpers.StatementCollector;
import org.semanticweb.owlapi.formats.RDFXMLOntologyFormat;
import org.semanticweb.owlapi.formats.RDFXMLOntologyFormatFactory;
import org.semanticweb.owlapi.formats.RioRDFOntologyFormat;
import org.semanticweb.owlapi.formats.RioRDFOntologyFormatFactory;
import org.semanticweb.owlapi.io.OWLParserFactory;
import org.semanticweb.owlapi.io.OWLParserFactoryRegistry;
import org.semanticweb.owlapi.io.StreamDocumentSource;
import org.semanticweb.owlapi.model.IRI;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyFormat;
import org.semanticweb.owlapi.model.OWLOntologyManager;
import org.semanticweb.owlapi.model.OWLOntologyManagerFactoryRegistry;
import org.semanticweb.owlapi.reasoner.InferenceType;
import org.semanticweb.owlapi.reasoner.OWLReasoner;
import org.semanticweb.owlapi.reasoner.OWLReasonerFactory;
import org.semanticweb.owlapi.reasoner.OWLReasonerFactoryRegistry;
import org.semanticweb.owlapi.rio.RioRenderer;
import org.semanticweb.owlapi.util.InferredOntologyGenerator;
import org.semanticweb.owlapi.vocab.PrefixOWLOntologyFormat;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.github.ansell.oas.objects.Ontology;
import com.github.ansell.oas.objects.impl.OntologyImpl;
import com.github.ansell.oas.utils.ContextInsensitiveStatementComparator;
import com.github.ansell.oas.utils.OasConstants;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class ParseOntologyTest
{
    private final Logger log = LoggerFactory.getLogger(ParseOntologyTest.class);
    
    private InputStream input;
    private String inputString;
    private ValueFactory valueFactory;
    
    /**
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() throws Exception
    {
        this.valueFactory = ValueFactoryImpl.getInstance();
        
        this.input = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        Assert.assertNotNull("Could not find test ontology file", this.input);
    }
    
    /**
     * @throws java.lang.Exception
     */
    @After
    public void tearDown() throws Exception
    {
        this.inputString = null;
        
        try
        {
            this.input.close();
        }
        finally
        {
            this.input = null;
        }
    }
    
    @Test
    public void testInferredAxioms() throws Exception
    {
        final RDFParser createParser = Rio.createParser(RDFFormat.RDFXML);
        
        final StatementCollector rdfStatementCollector = new StatementCollector();
        createParser.setRDFHandler(rdfStatementCollector);
        createParser.parse(this.input, "http://purl.obolibrary.org/obo/po.owl");
        
        Assert.assertEquals(44285, rdfStatementCollector.getStatements().size());
        
        final TreeSet<Statement> rdfParsedStatements =
                new TreeSet<Statement>(new ContextInsensitiveStatementComparator());
        Assert.assertTrue(rdfParsedStatements.addAll(rdfStatementCollector.getStatements()));
        // verify that there were no duplicated statements
        Assert.assertEquals(44285, rdfParsedStatements.size());
        
        final OWLOntologyManager testManager = OWLOntologyManagerFactoryRegistry.createOWLOntologyManager();
        // inputString = IOUtils.toString(input, "UTF-8");
        // Assert.assertEquals(4899469, inputString.length());
        
        final RioRDFOntologyFormatFactory formatFactory = new RDFXMLOntologyFormatFactory();
        
        // Need to do this to reset the stream in the simplest manner possible
        this.input = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test ontology file", this.input);
        
        final StreamDocumentSource documentSource =
                new StreamDocumentSource(this.input, IRI.create("http://purl.obolibrary.org/obo/po.owl"), formatFactory);
        
        final Ontology nextOntology =
                new OntologyImpl(this.valueFactory.createURI("http://purl.obolibrary.org/obo/po.owl"));
        
        final OWLOntology parsedOntology = testManager.createOntology(nextOntology.toOWLOntologyID());
        
        final OWLParserFactory parserFactory = OWLParserFactoryRegistry.getInstance().getParserFactory(formatFactory);
        
        Assert.assertNotNull("Could not find parser factory", parserFactory);
        
        final OWLOntologyFormat parsedFormat =
                parserFactory.createParser(testManager).parse(documentSource, parsedOntology);
        
        final String reasonerName = OasConstants.OAS_REASONER;
        
        final OWLReasonerFactory configuredReasoner =
                OWLReasonerFactoryRegistry.getInstance().getReasonerFactory(reasonerName);
        
        final StatementCollector inferredAxiomsStatementCollector =
                new StatementCollector(new ArrayList<Statement>(4096));
        
        Assert.assertNotNull("Could not find reasoner", configuredReasoner);
        
        final String inferredOntologyUri = "urn:oas:test:inferredontology:" + UUID.randomUUID().toString();
        
        final OWLOntology inferredAxiomsOntology = testManager.createOntology(IRI.create(inferredOntologyUri));
        final OWLReasoner reasoner = configuredReasoner.createReasoner(parsedOntology);
        
        Assert.assertTrue("Reasoner on ontology was not consistent", reasoner.isConsistent());
        
        reasoner.precomputeInferences(InferenceType.CLASS_HIERARCHY);
        
        // TODO: How often will an explicit list of InferredAxiomGenerator instances be
        // needed here?
        // Create an InferredOntologyGenerator using the default list of
        // InferredAxiomGenerators
        final InferredOntologyGenerator iog = new InferredOntologyGenerator(reasoner);
        
        // the original parsedOntology is already known inside of the reasoner since the
        // call to
        iog.fillOntology(testManager, inferredAxiomsOntology);
        
        // create a format that will be used purely to store prefixes, as all triples will
        // go to
        // the statementCollector (or other RDFHandler in future)
        final RioRDFOntologyFormat nextFormat = new RDFXMLOntologyFormat();
        
        // attempt to preserve the prefixes if they submitted the original ontology in a
        // prefix
        // capable format
        // NOTE: Storing the results as RDF Triples in a Repository may not persist these
        // prefixes, they will only be persisted if they are stored as a concrete RDF
        // serialisation, or if the prefixes are stored in the database separately and then
        // injected into an RDFHandler.handleNamespace method in future.
        // The prefixes will however be available in the statementCollector temporarily
        // FIXME: Determine the semantics of the SailRepositoryConnection namespace/prefix
        // handling methods that are being used to attempt to store these permanently
        // (without
        // overwriting previous prefixes)
        if(parsedFormat instanceof PrefixOWLOntologyFormat)
        {
            nextFormat.copyPrefixesFrom((PrefixOWLOntologyFormat)parsedFormat);
        }
        
        final RioRenderer inferredAxiomsRenderer =
                new RioRenderer(inferredAxiomsOntology, inferredAxiomsStatementCollector, nextFormat);
        inferredAxiomsRenderer.render();
        
        Assert.assertEquals(2994, inferredAxiomsStatementCollector.getStatements().size());
        
        if(this.log.isTraceEnabled())
        {
            for(final Statement nextStatement : inferredAxiomsStatementCollector.getStatements())
            {
                this.log.trace(nextStatement.toString());
            }
        }
        
        final TreeSet<Statement> inferredOwlStatements =
                new TreeSet<Statement>(new ContextInsensitiveStatementComparator());
        Assert.assertTrue(inferredOwlStatements.addAll(inferredAxiomsStatementCollector.getStatements()));
        // verify that there were no duplicated statements in the inferred statments
        Assert.assertEquals(2994, inferredOwlStatements.size());
        
        // FIXME: It seems like it is acceptable for the inferred statements to contain statements
        // from the direct parsed statements
        // check that there is no overlap between inferred and direct RDF statements
        // for(Statement nextInferredStatement : inferredOwlStatements)
        // {
        // Assert.assertFalse(
        // "Direct RDF parsed statements contained an inferred statement: " +
        // nextInferredStatement.toString(),
        // rdfParsedStatements.contains(nextInferredStatement));
        // }
        //
        // for(Statement nextRdfParsedStatement : rdfParsedStatements)
        // {
        // Assert.assertFalse(
        // "Inferred statements contained a direct rdf parsed statement: " +
        // nextRdfParsedStatement.toString(),
        // inferredOwlStatements.contains(nextRdfParsedStatement));
        // }
        
    }
    
    @Test
    public void testParse() throws Exception
    {
        // inputString = IOUtils.toString(input, "UTF-8");
        // Assert.assertEquals(4899469, inputString.length());
        
        final RDFParser createParser = Rio.createParser(RDFFormat.RDFXML);
        
        final StatementCollector rdfStatementCollector = new StatementCollector();
        createParser.setRDFHandler(rdfStatementCollector);
        createParser.parse(this.input, "http://test.example.org/base/uri/");
        
        Assert.assertEquals(44285, rdfStatementCollector.getStatements().size());
        
        final TreeSet<Statement> rdfParsedStatements =
                new TreeSet<Statement>(new ContextInsensitiveStatementComparator());
        Assert.assertTrue(rdfParsedStatements.addAll(rdfStatementCollector.getStatements()));
        // verify that there were no duplicated statements
        Assert.assertEquals(44285, rdfParsedStatements.size());
    }
    
}
