/**
 * 
 */
package com.github.ansell.oas.test;

import java.io.StringWriter;

import org.openrdf.model.Model;
import org.openrdf.model.Resource;
import org.openrdf.model.impl.LinkedHashModel;
import org.openrdf.model.impl.ValueFactoryImpl;
import org.openrdf.model.vocabulary.RDF;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.Rio;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.github.ansell.oas.utils.OasConstants;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public class TestUtils extends RestletTestUtils
{
    private static final Logger log = LoggerFactory.getLogger(TestUtils.class);
    
    /**
     * 
     * @param outputFormat
     * @param annotationUri
     * @param taggedResourceUri
     * @param taggedResourceType
     * @param ontologyUri
     * @param tagName
     * @param authorUri
     * @return
     * @throws RDFHandlerException
     */
    public static String createTestAnnotationDocument(final String outputFormat, final String annotationUri,
            final String taggedResourceUri, final String taggedResourceType, final String ontologyUri,
            final String tagName, final String authorUri) throws RDFHandlerException
    {
        final ValueFactoryImpl vf = ValueFactoryImpl.getInstance();
        final RDFFormat format = Rio.getWriterFormatForMIMEType(outputFormat);
        final Model model = new LinkedHashModel();
        final StringWriter output = new StringWriter();
        
        Resource annotationIdentifier;
        
        if(annotationUri == null || annotationUri.equals(""))
        {
            annotationIdentifier = vf.createBNode();
        }
        else
        {
            annotationIdentifier = vf.createURI(annotationUri);
        }
        
        model.add(annotationIdentifier, RDF.TYPE, OasConstants.TAGGING_TAGGING);
        model.add(annotationIdentifier, OasConstants.TAGGING_TAGGED_RESOURCE, vf.createURI(taggedResourceUri));
        if(authorUri != null)
        {
            model.add(annotationIdentifier, OasConstants.FOAF_MAKER, vf.createURI(authorUri));
        }
        
        model.add(annotationIdentifier, OasConstants.TAGGING_NAME, vf.createLiteral(tagName));
        model.add(annotationIdentifier, OasConstants.MOAT_TAG_MEANING, vf.createURI(ontologyUri));
        if(taggedResourceType != null)
        {
            model.add(vf.createURI(taggedResourceUri), RDF.TYPE, vf.createURI(taggedResourceType));
        }
        
        if(format != null)
        {
            Rio.write(model, output, format);
        }
        else
        {
            TestUtils.log.error("Did not recognise outputFormat in createTestDocument: {}", outputFormat);
            throw new IllegalArgumentException("Did not recognise outputFormat in createTestDocument: " + outputFormat);
        }
        
        return output.toString();
    }
    
    /**
     * Creates a new test ontology document using the given format.
     * 
     * @param outputFormat
     *            The format to generate the test document using.
     * @param ontologyUri
     *            The URI for the ontology to generate.
     * @param ontologyVersionUri
     *            The version URI for the ontology to be generated.
     * @return A string containing a test ontology serialised in the given output format.
     */
    public static String createTestOntologyDocument(final String outputFormat, final String ontologyUri,
            final String ontologyVersionUri)
    {
        final StringBuilder result = new StringBuilder();
        
        if(outputFormat.equals("application/rdf+xml"))
        {
            result.append("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n").append("<rdf:RDF \n")
                    .append("xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" \n")
                    .append("  xmlns:owl=\"http://www.w3.org/2002/07/owl#\" \n").append(" > \n");
            
            if(ontologyUri == null || ontologyUri.equals(""))
            {
                result.append("<rdf:Description> \n");
            }
            else
            {
                result.append("<rdf:Description rdf:about=\"" + ontologyUri + "\"> \n");
            }
            
            result.append("  <rdf:type rdf:resource=\"http://www.w3.org/2002/07/owl#Ontology\"/> \n")
                    .append("  <owl:versionIRI rdf:resource=\"" + ontologyVersionUri + "\"/> \n")
                    .append("</rdf:Description> \n").append("</rdf:RDF>\n");
        }
        else if(outputFormat.equals("application/json"))
        {
            if(ontologyUri == null || ontologyUri.equals(""))
            {
                result.append("{\"_:abc123\": \n");
            }
            else
            {
                result.append("{\"" + ontologyUri + "\": \n");
            }
            result.append("{ \"http://www.w3.org/1999/02/22-rdf-syntax-ns#type\": ").append("[")
                    .append("{ \"value\": \"http://www.w3.org/2002/07/owl#Ontology\", \"type\": \"uri\" }")
                    .append("], \"http://www.w3.org/2002/07/owl#versionIRI\": ").append("[")
                    .append("{ \"value\": \"" + ontologyVersionUri + "\", \"type\": \"uri\" }").append("]").append("}")
                    .append("}");
        }
        else
        {
            throw new IllegalArgumentException("Did not recognise outputFormat in createTestDocument");
        }
        
        return result.toString();
    }
    
    public static String createTestUriDirectCountDocument(final String outputFormat, final String uriCountImplUri,
            final int count)
    {
        final StringBuilder result = new StringBuilder();
        
        result.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        result.append("<rdf:RDF");
        result.append(" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">");
        if(uriCountImplUri == null || uriCountImplUri.equals(""))
        {
            result.append("<rdf:Description> \n");
        }
        else
        {
            result.append("<rdf:Description rdf:about=\"" + uriCountImplUri + "\"> \n");
        }
        result.append("<rdf:type rdf:resource=\"urn:tag:UriCountType\"/>");
        result.append("<uriDirectCount xmlns=\"http://purl.org/oas/\" rdf:datatype=\"http://www.w3.org/2001/XMLSchema#int\">");
        result.append(Integer.toString(count));
        result.append("</uriDirectCount>");
        result.append("</rdf:Description>");
        result.append("</rdf:RDF>");
        
        return result.toString();
    }
    
    public static String createTestUriRecursiveCountDocument(final String outputFormat, final String uriCountImplUri,
            final int count)
    {
        final StringBuilder result = new StringBuilder();
        
        result.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        result.append("<rdf:RDF");
        result.append(" xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">");
        if(uriCountImplUri == null || uriCountImplUri.equals(""))
        {
            result.append("<rdf:Description> \n");
        }
        else
        {
            result.append("<rdf:Description rdf:about=\"" + uriCountImplUri + "\"> \n");
        }
        result.append("<rdf:type rdf:resource=\"urn:tag:UriCountType\"/>");
        result.append("<uriRecursiveCount xmlns=\"http://purl.org/oas/\" rdf:datatype=\"http://www.w3.org/2001/XMLSchema#int\">");
        result.append(Integer.toString(count));
        result.append("</uriRecursiveCount>");
        result.append("</rdf:Description>");
        result.append("</rdf:RDF>");
        
        return result.toString();
    }
    
}
