/**
 * 
 */
package com.github.ansell.oas.utils.test;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.openrdf.model.Literal;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.ValueFactory;
import org.openrdf.model.vocabulary.OWL;
import org.openrdf.model.vocabulary.RDF;
import org.openrdf.model.vocabulary.RDFS;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryConnection;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.RDFParseException;
import org.openrdf.rio.RDFParser;
import org.openrdf.rio.Rio;
import org.openrdf.rio.helpers.StatementCollector;
import org.openrdf.sail.memory.MemoryStore;
import org.restlet.data.MediaType;
import org.semanticweb.owlapi.profiles.OWLProfile;
import org.semanticweb.owlapi.profiles.OWLProfileRegistry;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.github.ansell.oas.objects.Annotation;
import com.github.ansell.oas.objects.Ontology;
import com.github.ansell.oas.objects.ResourceCount;
import com.github.ansell.oas.objects.impl.AnnotationImpl;
import com.github.ansell.oas.objects.impl.OntologyImpl;
import com.github.ansell.oas.objects.impl.UriCountImpl;
import com.github.ansell.oas.test.TestUtils;
import com.github.ansell.oas.utils.OasConstants;
import com.github.ansell.oas.utils.SesameUtils;

/**
 * Test class for the Sesame Utilities class
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public final class SesameUtilsTest
{
    private static final Logger LOG = LoggerFactory.getLogger(SesameUtilsTest.class);
    
    private Repository testRepository;
    private RepositoryConnection testRepositoryConnection;
    private ValueFactory testValueFactory;
    
    private Annotation testAnnotation;
    private Ontology testOntology;
    private ResourceCount testResourceCount;
    
    private URI testAnnotationUri1;
    private URI testAnnotationObjectTypeUri1;
    private URI testAnnotationObjectUri1;
    private URI testAnnotationOntologyUri1;
    private URI testAnnotationOntologyTermUri1;
    private URI testAnnotationAnnotatorUri1;
    private Literal testAnnotationTagLiteral1;
    
    private URI testResourceCountUri1;
    
    private URI testOntologyUri1;
    private URI testOntologyVersionUri1;
    private URI testOntologyInferredAxiomsUri1;
    
    private List<OWLProfile> testAllProfiles;
    private List<OWLProfile> testEmptyProfiles;
    
    private String testBaseUri;
    
    /**
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() throws Exception
    {
        this.testRepository = new SailRepository(new MemoryStore());
        this.testRepository.initialize();
        
        this.testRepositoryConnection = this.testRepository.getConnection();
        
        this.testValueFactory = this.testRepository.getValueFactory();
        
        this.testBaseUri = "urn:test:";
        
        this.testAnnotationUri1 = this.testValueFactory.createURI("urn:test:annotation:1");
        
        this.testAnnotationObjectTypeUri1 = this.testValueFactory.createURI("urn:test:annotation:objecttype:1");
        this.testAnnotationObjectUri1 = this.testValueFactory.createURI("urn:test:annotation:object:1");
        this.testAnnotationAnnotatorUri1 = this.testValueFactory.createURI("urn:test:annotation:annotator:1");
        this.testAnnotationOntologyUri1 = this.testValueFactory.createURI("urn:test:annotation:ontology:1");
        this.testAnnotationOntologyTermUri1 = this.testValueFactory.createURI("urn:test:annotation:ontologyterm:1");
        this.testAnnotationTagLiteral1 = this.testValueFactory.createLiteral("test tag number 1");
        
        this.testResourceCountUri1 = this.testValueFactory.createURI("urn:test:resourcecount:1");
        
        this.testOntologyUri1 = this.testValueFactory.createURI("urn:test:ontology:1");
        this.testOntologyVersionUri1 = this.testValueFactory.createURI("urn:test:ontology:1:version:1");
        this.testOntologyInferredAxiomsUri1 = this.testValueFactory.createURI("urn:test:ontology:inferredAxioms:1");
        
        this.testAllProfiles = new ArrayList<OWLProfile>();
        this.testAllProfiles.addAll(OWLProfileRegistry.getInstance().getProfiles());
        
        this.testEmptyProfiles = Collections.emptyList();
    }
    
    /**
     * Shuts down the repository, releasing any resources that it was using.
     * 
     * @throws java.lang.Exception
     */
    @After
    public void tearDown() throws Exception
    {
        this.testAnnotationUri1 = null;
        this.testAnnotationAnnotatorUri1 = null;
        this.testAnnotationObjectTypeUri1 = null;
        this.testAnnotationObjectUri1 = null;
        this.testAnnotationOntologyTermUri1 = null;
        this.testAnnotationOntologyUri1 = null;
        this.testAnnotationTagLiteral1 = null;
        
        this.testResourceCountUri1 = null;
        
        this.testOntologyUri1 = null;
        this.testOntologyVersionUri1 = null;
        this.testOntologyInferredAxiomsUri1 = null;
        
        this.testAnnotation = null;
        this.testOntology = null;
        this.testResourceCount = null;
        
        this.testValueFactory = null;
        
        if(this.testRepositoryConnection != null)
        {
            try
            {
                this.testRepositoryConnection.close();
            }
            catch(final RepositoryException e)
            {
                SesameUtilsTest.LOG.error("Found repository exception in test tearDown", e);
            }
        }
        
        this.testRepositoryConnection = null;
        
        if(this.testRepository != null)
        {
            try
            {
                this.testRepository.shutDown();
            }
            catch(final RepositoryException e)
            {
                SesameUtilsTest.LOG.error("Found repository exception in test tearDown", e);
            }
        }
        
        this.testRepository = null;
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.utils.SesameUtils#createAnnotationByUriFromRepository(org.openrdf.model.URI, org.openrdf.repository.Repository)}
     * .
     */
    @Test
    public final void testCreateAnnotationByUriFromRepository()
    {
        try
        {
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1, RDF.TYPE,
                    OasConstants.TAGGING_TAGGING), SesameUtils.ANNOTATION_MANAGEMENT_GRAPH);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1, RDF.TYPE,
                    OasConstants.TAGGING_TAGGING), this.testAnnotationUri1);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1,
                    OasConstants.FOAF_MAKER, this.testAnnotationAnnotatorUri1), this.testAnnotationUri1);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1,
                    OasConstants.MOAT_TAG_MEANING, this.testAnnotationOntologyTermUri1), this.testAnnotationUri1);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1,
                    OasConstants.TAGGING_NAME, this.testAnnotationTagLiteral1), this.testAnnotationUri1);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1,
                    OasConstants.TAGGING_TAGGED_RESOURCE, this.testAnnotationObjectUri1), this.testAnnotationUri1);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationObjectUri1,
                    RDF.TYPE, this.testAnnotationObjectTypeUri1), this.testAnnotationUri1);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(
                    this.testAnnotationOntologyTermUri1, RDFS.ISDEFINEDBY, this.testAnnotationOntologyUri1),
                    this.testAnnotationUri1);
            
            this.testRepositoryConnection.commit();
            
            this.testAnnotation =
                    SesameUtils.createAnnotationByUriFromRepository(this.testAnnotationUri1, this.testRepository,
                            SesameUtils.ANNOTATION_MANAGEMENT_GRAPH);
            
            Assert.assertNotNull("Created annotation was null", this.testAnnotation);
            
            Assert.assertEquals("Annotation URI did not match", this.testAnnotationUri1,
                    this.testAnnotation.getAnnotationUniqueId());
            Assert.assertEquals("Annotated Object URI did not match", this.testAnnotationObjectUri1,
                    this.testAnnotation.getAnnotatedObjectUri());
            Assert.assertEquals("Annotated Object Type URI did not match", this.testAnnotationObjectTypeUri1,
                    this.testAnnotation.getAnnotatedObjectType());
            Assert.assertEquals("Annotator did not match", this.testAnnotationAnnotatorUri1,
                    this.testAnnotation.getAnnotator());
            Assert.assertEquals("Ontology Term URI did not match", this.testAnnotationOntologyTermUri1,
                    this.testAnnotation.getOntologyTermUri());
            Assert.assertEquals("Annotation Tag did not match", this.testAnnotationTagLiteral1,
                    this.testAnnotation.getTag());
            Assert.assertEquals("Ontology URI did not match", this.testAnnotationOntologyUri1,
                    this.testAnnotation.getOntologyUri());
        }
        catch(final RepositoryException ex)
        {
            Assert.fail("Found unexpected RepositoryException");
        }
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.utils.SesameUtils#createAnnotationsFromRepository(org.openrdf.repository.Repository, URI)}
     * .
     */
    @Test
    public final void testCreateAnnotationsFromRepository()
    {
        try
        {
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1, RDF.TYPE,
                    OasConstants.TAGGING_TAGGING), SesameUtils.ANNOTATION_MANAGEMENT_GRAPH);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1, RDF.TYPE,
                    OasConstants.TAGGING_TAGGING), this.testAnnotationUri1);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1,
                    OasConstants.FOAF_MAKER, this.testAnnotationAnnotatorUri1, this.testAnnotationUri1));
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1,
                    OasConstants.MOAT_TAG_MEANING, this.testAnnotationOntologyTermUri1), this.testAnnotationUri1);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1,
                    OasConstants.TAGGING_NAME, this.testAnnotationTagLiteral1), this.testAnnotationUri1);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationUri1,
                    OasConstants.TAGGING_TAGGED_RESOURCE, this.testAnnotationObjectUri1), this.testAnnotationUri1);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(this.testAnnotationObjectUri1,
                    RDF.TYPE, this.testAnnotationObjectTypeUri1), this.testAnnotationUri1);
            this.testRepositoryConnection.add(this.testValueFactory.createStatement(
                    this.testAnnotationOntologyTermUri1, RDFS.ISDEFINEDBY, this.testAnnotationOntologyUri1),
                    this.testAnnotationUri1);
            
            // TODO: create more objects to test multiple objects support in this method
            
            this.testRepositoryConnection.commit();
            
            final Map<URI, Annotation> createdAnnotations =
                    SesameUtils.createAnnotationsFromRepository(this.testRepository, this.testBaseUri,
                            SesameUtils.ANNOTATION_MANAGEMENT_GRAPH);
            
            Assert.assertNotNull("Created annotations map was null", createdAnnotations);
            Assert.assertEquals("The results did not contain exactly 1 newly created annotation", 1,
                    createdAnnotations.size());
            Assert.assertTrue("Results did not contain the test annotation",
                    createdAnnotations.containsKey(this.testAnnotationUri1));
            
            final Annotation createdAnnotation = createdAnnotations.get(this.testAnnotationUri1);
            
            Assert.assertNotNull("Created annotation was null", createdAnnotation);
            
            Assert.assertEquals("Annotation URI did not match", this.testAnnotationUri1,
                    createdAnnotation.getAnnotationUniqueId());
            Assert.assertEquals("Annotated Object URI did not match", this.testAnnotationObjectUri1,
                    createdAnnotation.getAnnotatedObjectUri());
            Assert.assertEquals("Annotated Object Type URI did not match", this.testAnnotationObjectTypeUri1,
                    createdAnnotation.getAnnotatedObjectType());
            Assert.assertEquals("Annotator did not match", this.testAnnotationAnnotatorUri1,
                    createdAnnotation.getAnnotator());
            Assert.assertEquals("Ontology Term URI did not match", this.testAnnotationOntologyTermUri1,
                    createdAnnotation.getOntologyTermUri());
            Assert.assertEquals("Annotation Tag did not match", this.testAnnotationTagLiteral1,
                    createdAnnotation.getTag());
            Assert.assertEquals("Ontology URI did not match", this.testAnnotationOntologyUri1,
                    createdAnnotation.getOntologyUri());
        }
        catch(final RepositoryException ex)
        {
            Assert.fail("Found unexpected RepositoryException");
        }
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.utils.SesameUtils#parseAnnotations(java.io.Reader, java.lang.String, java.lang.String)}
     * .
     */
    @Test
    public final void testCreateAnnotationUsingRDF() throws Exception
    {
        final StringReader testReader =
                new StringReader(TestUtils.createTestAnnotationDocument("application/rdf+xml",
                        this.testAnnotationUri1.stringValue(), this.testAnnotationObjectUri1.stringValue(),
                        this.testAnnotationObjectTypeUri1.stringValue(),
                        this.testAnnotationOntologyTermUri1.stringValue(),
                        this.testAnnotationTagLiteral1.stringValue(), this.testAnnotationAnnotatorUri1.stringValue()));
        
        final Map<URI, Annotation> map =
                SesameUtils.parseAnnotations(testReader, "http://error.example.org/", "application/rdf+xml");
        
        Assert.assertEquals(1, map.size());
        
        this.testAnnotation = map.values().iterator().next();
        
        Assert.assertNotNull("Created annotation was null", this.testAnnotation);
        
        Assert.assertEquals("Annotation URI did not match", this.testAnnotationUri1,
                this.testAnnotation.getAnnotationUniqueId());
        Assert.assertEquals("Annotated Object URI did not match", this.testAnnotationObjectUri1,
                this.testAnnotation.getAnnotatedObjectUri());
        Assert.assertEquals("Annotated Object Type URI did not match", this.testAnnotationObjectTypeUri1,
                this.testAnnotation.getAnnotatedObjectType());
        Assert.assertEquals("Annotator did not match", this.testAnnotationAnnotatorUri1,
                this.testAnnotation.getAnnotator());
        Assert.assertEquals("Ontology URI did not match", this.testAnnotationOntologyTermUri1,
                this.testAnnotation.getOntologyTermUri());
        Assert.assertEquals("Annotation Tag did not match", this.testAnnotationTagLiteral1,
                this.testAnnotation.getTag());
    }
    
    @Test
    public final void testCreateOntologiesFromRepository() throws Exception
    {
        final StringReader testDocument =
                new StringReader(TestUtils.createTestOntologyDocument(MediaType.APPLICATION_RDF_XML.getName(),
                        "http://test.example.org/test/create/ontologies/1",
                        "http://test.example.org/test/create/ontologies/1.1"));
        
        this.testRepositoryConnection.add(testDocument, "http://base.example.org/test/base/uri/1/", RDFFormat.RDFXML);
        
        final Map<URI, List<Ontology>> map = SesameUtils.createOntologiesFromRepository(this.testRepository);
        
        Assert.assertEquals(1, map.size());
        Assert.assertEquals(1, map.keySet().size());
        Assert.assertEquals(1, map.values().size());
        
        for(final URI nextUri : map.keySet())
        {
            Assert.assertNotNull(map.get(nextUri));
            
            Assert.assertEquals(1, map.get(nextUri).size());
        }
    }
    
    @Test
    public final void testCreateOntologiesUsingRDF() throws Exception
    {
        final StringReader testDocument =
                new StringReader(TestUtils.createTestOntologyDocument(MediaType.APPLICATION_RDF_XML.getName(),
                        "http://test.example.org/test/create/ontologies/1",
                        "http://test.example.org/test/create/ontologies/1.1"));
        
        final Map<URI, List<Ontology>> map =
                SesameUtils.parseOntologies(testDocument, "http://base.example.org/test/base/uri/1/",
                        MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, map.size());
        Assert.assertEquals(1, map.keySet().size());
        Assert.assertEquals(1, map.values().size());
        
        for(final URI nextUri : map.keySet())
        {
            Assert.assertNotNull(map.get(nextUri));
            
            Assert.assertEquals(1, map.get(nextUri).size());
        }
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.utils.SesameUtils#parseUriCounts(java.io.Reader, java.lang.String, java.lang.String)}
     * .
     */
    @Test
    public final void testCreateUriCountUsingRDF()
    {
        final StringReader testReader =
                new StringReader(TestUtils.createTestUriDirectCountDocument("application/rdf+xml",
                        this.testResourceCountUri1.stringValue(), 42));
        
        this.testResourceCount =
                SesameUtils.parseUriCounts(testReader, "http://error.example.org/", "application/rdf+xml");
        
        Assert.assertNotNull("Created resource count was null", this.testResourceCount);
        
        // TODO: should counts have URIs?
        Assert.assertEquals("Resource count did not match", 42, this.testResourceCount.getCount());
    }
    
    @Test
    public void testGenerateNewOntologyVersionUriAllNull()
    {
        try
        {
            SesameUtils.generateNewOntologyVersionUri(null, null, null);
            
            Assert.fail("Did not find expected IllegalArgumentException");
        }
        catch(final IllegalArgumentException iae)
        {
            Assert.assertTrue(iae.getMessage().contains("Ontology URI and ontology version URI must not both be null"));
        }
        catch(final RepositoryException e)
        {
            Assert.fail("Found repository exception when we were expecting an IllegalArgumentException");
        }
    }
    
    @Test
    public void testGenerateNewOntologyVersionUriConflictBothOntologyAndVersionUri()
    {
        try
        {
            final URI newOntologyVersionUri =
                    SesameUtils.generateNewOntologyVersionUri(this.testOntologyUri1, this.testOntologyVersionUri1,
                            this.testRepository, SesameUtils.ONTOLOGY_MANAGER_GRAPH);
            
            Assert.assertNotNull(newOntologyVersionUri);
            Assert.assertNotSame(newOntologyVersionUri, this.testOntologyUri1);
            Assert.assertEquals(newOntologyVersionUri, this.testOntologyVersionUri1);
            
            this.testOntology = new OntologyImpl();
            this.testOntology.setOntologyUri(this.testOntologyUri1);
            this.testOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
            
            // Store the ontology so that the repository will have knowledge of it
            final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
            
            this.testRepository =
                    SesameUtils.storeOntology(this.testOntology, new InputStreamReader(testInputStream, "UTF-8"),
                            "http://test.example.org/test/1/base/", RDFFormat.RDFXML, this.testRepository,
                            SesameUtils.ONTOLOGY_MANAGER_GRAPH);
            
            final URI secondOntologyVersionUri =
                    SesameUtils.generateNewOntologyVersionUri(this.testOntologyUri1, newOntologyVersionUri,
                            this.testRepository, SesameUtils.ONTOLOGY_MANAGER_GRAPH);
            
            Assert.assertNotNull(secondOntologyVersionUri);
            Assert.assertNotSame(secondOntologyVersionUri, this.testOntologyUri1);
            Assert.assertNotSame(secondOntologyVersionUri, newOntologyVersionUri);
            Assert.assertNotSame(secondOntologyVersionUri, this.testOntologyVersionUri1);
        }
        catch(final RepositoryException e)
        {
            Assert.fail("Found repository exception while attempting to generate new ontology version URI");
        }
        catch(final RDFParseException e)
        {
            Assert.fail("Found rdf parse exception while attempting to generate new ontology version URI");
        }
        catch(final IOException e)
        {
            Assert.fail("Found io exception while attempting to generate new ontology version URI");
        }
        catch(final RDFHandlerException e)
        {
            Assert.fail("Found rdf handler exception while attempting to generate new ontology version URI");
        }
    }
    
    @Test
    public void testGenerateNewOntologyVersionUriSimpleBothOntologyAndVersionUri()
    {
        try
        {
            final URI newOntologyVersionUri =
                    SesameUtils.generateNewOntologyVersionUri(this.testOntologyUri1, this.testOntologyVersionUri1,
                            this.testRepository, SesameUtils.ONTOLOGY_MANAGER_GRAPH);
            
            Assert.assertNotNull(newOntologyVersionUri);
            Assert.assertNotSame(newOntologyVersionUri, this.testOntologyUri1);
            // NOTE: The SesameUtils implementation is known here, so we are expecting this to be
            // true. In AbstractOntologyTest we cannot assume this equivalence, as the
            // implementation may be different.
            Assert.assertEquals(newOntologyVersionUri, this.testOntologyVersionUri1);
        }
        catch(final RepositoryException e)
        {
            Assert.fail("Found repository exception while attempting to generate new ontology version URI");
        }
    }
    
    @Test
    public void testGenerateNewOntologyVersionUriSimpleOnlyOntologyUri()
    {
        try
        {
            final URI newOntologyVersionUri =
                    SesameUtils.generateNewOntologyVersionUri(this.testOntologyUri1, null, this.testRepository,
                            SesameUtils.ONTOLOGY_MANAGER_GRAPH);
            
            Assert.assertNotNull(newOntologyVersionUri);
            Assert.assertNotSame(newOntologyVersionUri, this.testOntologyUri1);
            
            // NOTE: WhiteBox test We know the implementation of SesameUtils, so we are testing it
            // here
            Assert.assertEquals(newOntologyVersionUri,
                    this.testValueFactory.createURI(OasConstants.AUTOGENERATED_NS, this.testOntologyUri1.stringValue()));
        }
        catch(final RepositoryException e)
        {
            Assert.fail("Found repository exception while attempting to generate new ontology version URI");
        }
    }
    
    @Test
    public void testGenerateNewOntologyVersionUriSimpleOnlyOntologyVersionUri()
    {
        try
        {
            final URI newOntologyVersionUri =
                    SesameUtils.generateNewOntologyVersionUri(null, this.testOntologyVersionUri1, this.testRepository,
                            SesameUtils.ONTOLOGY_MANAGER_GRAPH);
            
            Assert.assertNotNull(newOntologyVersionUri);
            Assert.assertNotSame(newOntologyVersionUri, this.testOntologyUri1);
            Assert.assertEquals(newOntologyVersionUri, this.testOntologyVersionUri1);
        }
        catch(final RepositoryException e)
        {
            Assert.fail("Found repository exception while attempting to generate new ontology version URI");
        }
    }
    
    @Test
    public void testGenerateNextURIEndsWithOne()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:1");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:2", nextURI.stringValue());
    }
    
    @Test
    public void testGenerateNextURIEndsWithSingleNine()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:9");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:10", nextURI.stringValue());
    }
    
    @Test
    public void testGenerateNextURIEndsWithSingleNineNine()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:99");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:100", nextURI.stringValue());
    }
    
    @Test
    public void testGenerateNextURIEndsWithSingleNineNineNine()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:999");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:1000", nextURI.stringValue());
    }
    
    @Test
    public void testGenerateNextURIEndsWithTwo()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:2");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:3", nextURI.stringValue());
    }
    
    @Test
    public void testGenerateNextURIEndsWithZero()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:0");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:1", nextURI.stringValue());
    }
    
    @Test
    public void testGenerateNextURIEndsWithZeroNine()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:09");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:10", nextURI.stringValue());
    }
    
    @Test
    public void testGenerateNextURIEndsWithZeroNineNine()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:099");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:100", nextURI.stringValue());
    }
    
    @Test
    public void testGenerateNextURIEndsWithZeroOneNine()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:019");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:020", nextURI.stringValue());
    }
    
    @Test
    public void testGenerateNextURIEndsWithZeroZeroOneNine()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:0019");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:0020", nextURI.stringValue());
    }
    
    @Test
    public void testGenerateNextURIEndsWithZeroZeroZeroZero()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:0000");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:0001", nextURI.stringValue());
    }
    
    @Test
    public void testGenerateNextURIEndsWithZeroZeroZeroZeroNine()
    {
        final URI managedOntologyVersionUri = this.testValueFactory.createURI("urn:test:00009");
        
        final URI nextURI = SesameUtils.generateNextURI(managedOntologyVersionUri, this.testValueFactory);
        
        Assert.assertEquals("urn:test:00010", nextURI.stringValue());
    }
    
    @Test
    public void testStoreOntologyEmptyOntology() throws RepositoryException, RDFParseException, IOException,
        RDFHandlerException
    {
        this.testOntology = new OntologyImpl();
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        try
        {
            SesameUtils.storeOntology(this.testOntology, new InputStreamReader(testInputStream, "UTF-8"),
                    "http://test.base.example.org/test/1/", RDFFormat.RDFXML, this.testRepository,
                    SesameUtils.ONTOLOGY_MANAGER_GRAPH);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final IllegalArgumentException iae)
        {
            Assert.assertTrue(iae.getMessage().contains("Ontology URI must not be null"));
        }
        
    }
    
    @Test
    public void testStoreOntologyInferredEmptyWithVersionTrue() throws RepositoryException, RDFParseException,
        IOException, RDFHandlerException
    {
        this.testOntology = new OntologyImpl();
        this.testOntology.setOntologyUri(this.testValueFactory.createURI("http://test.example.org/ontology/0139"));
        this.testOntology.setOntologyVersionUri(this.testValueFactory
                .createURI("http://test.example.org/ontology/0139/version:1"));
        this.testOntology.setInferredOntologyUri(this.testOntologyInferredAxiomsUri1);
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        Assert.assertEquals(0, this.testRepositoryConnection.size());
        
        final StatementCollector statementCollector = new StatementCollector();
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFXML);
        parser.setRDFHandler(statementCollector);
        parser.parse(new InputStreamReader(testInputStream, "UTF-8"), "http://test.base.example.org/test/1/");
        
        // Empty inferred statements, to make sure the inferred axioms ontology reference is still
        // created
        
        // The rationale for this is that there may be many cases where there are no inferred
        // axioms, and the reference should be created nonetheless
        final ArrayList<Statement> emptyInferredStatements = new ArrayList<Statement>(0);
        
        final Repository storeOntology =
                SesameUtils.storeOntology(this.testOntology, statementCollector.getStatements(),
                        statementCollector.getNamespaces(), emptyInferredStatements, this.testRepository,
                        SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        // we passed in a repository so we should get the same repository object out in the
        // SesameUtils implementation.
        // Note: This is not necessary for the DataHandler contract so do not do these tests in
        // DataHandler tests
        Assert.assertEquals(storeOntology, this.testRepository);
        
        // final URI managedOntologyVersionUri =
        this.testOntology.ensureVersionUriUniqueness(this.testRepository, false, true,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        Assert.assertNotNull(this.testOntology.getOntologyVersionUri());
        
        Assert.assertEquals(10, this.testRepositoryConnection.size());
        Assert.assertEquals(3, this.testRepositoryConnection.size(this.testOntology.getOntologyVersionUri()));
        Assert.assertEquals(2, this.testRepositoryConnection.size(this.testOntologyInferredAxiomsUri1));
        Assert.assertEquals(5, this.testRepositoryConnection.size(SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OWL.VERSIONINFO, this.testValueFactory.createLiteral(this.testOntology.getOntologyVersionUri()
                        .stringValue(), OasConstants.XSD_ANYURI), false, this.testOntology.getOntologyVersionUri()));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntologyInferredAxiomsUri1, RDF.TYPE,
                OWL.ONTOLOGY, false, this.testOntologyInferredAxiomsUri1));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntologyInferredAxiomsUri1, OWL.IMPORTS,
                this.testOntology.getOntologyVersionUri(), false, this.testOntologyInferredAxiomsUri1));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OMV_CURRENTVERSION, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OAS_CURRENTINFERREDAXIOMSONTOLOGY, this.testOntologyInferredAxiomsUri1, false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyVersionUri(),
                OasConstants.OAS_INFERREDAXIOMSONTOLOGY, this.testOntologyInferredAxiomsUri1, false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
    }
    
    @Test
    public void testStoreOntologyInferredTrivialWithVersionTrue() throws RepositoryException, RDFParseException,
        IOException, RDFHandlerException
    {
        this.testOntology = new OntologyImpl();
        this.testOntology.setOntologyUri(this.testValueFactory.createURI("http://test.example.org/ontology/0139"));
        this.testOntology.setOntologyVersionUri(this.testValueFactory
                .createURI("http://test.example.org/ontology/0139/version:1"));
        this.testOntology.setInferredOntologyUri(this.testOntologyInferredAxiomsUri1);
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        Assert.assertEquals(0, this.testRepositoryConnection.size());
        
        final StatementCollector statementCollector = new StatementCollector();
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFXML);
        parser.setRDFHandler(statementCollector);
        parser.parse(new InputStreamReader(testInputStream, "UTF-8"), "http://test.base.example.org/test/1/");
        
        final ArrayList<Statement> nonEmptyInferredStatements = new ArrayList<Statement>(1);
        nonEmptyInferredStatements.add(this.testValueFactory.createStatement(this.testOntologyInferredAxiomsUri1,
                RDF.TYPE, OWL.ONTOLOGY));
        
        final Repository storeOntology =
                SesameUtils.storeOntology(this.testOntology, statementCollector.getStatements(),
                        statementCollector.getNamespaces(), nonEmptyInferredStatements, this.testRepository,
                        SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        // we passed in a repository so we should get the same repository object out in the
        // SesameUtils implementation.
        // Note: This is not necessary for the DataHandler contract so do not do these tests in
        // DataHandler tests
        Assert.assertEquals(storeOntology, this.testRepository);
        
        // final URI managedOntologyVersionUri =
        this.testOntology.ensureVersionUriUniqueness(this.testRepository, false, true,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        Assert.assertNotNull(this.testOntology.getOntologyVersionUri());
        
        Assert.assertEquals(10, this.testRepositoryConnection.size());
        Assert.assertEquals(3, this.testRepositoryConnection.size(this.testOntology.getOntologyVersionUri()));
        Assert.assertEquals(2, this.testRepositoryConnection.size(this.testOntologyInferredAxiomsUri1));
        Assert.assertEquals(5, this.testRepositoryConnection.size(SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OWL.VERSIONINFO, this.testValueFactory.createLiteral(this.testOntology.getOntologyVersionUri()
                        .stringValue(), OasConstants.XSD_ANYURI), false, this.testOntology.getOntologyVersionUri()));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntologyInferredAxiomsUri1, RDF.TYPE,
                OWL.ONTOLOGY, false, this.testOntologyInferredAxiomsUri1));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntologyInferredAxiomsUri1, OWL.IMPORTS,
                this.testOntology.getOntologyVersionUri(), false, this.testOntologyInferredAxiomsUri1));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OMV_CURRENTVERSION, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OAS_CURRENTINFERREDAXIOMSONTOLOGY, this.testOntologyInferredAxiomsUri1, false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyVersionUri(),
                OasConstants.OAS_INFERREDAXIOMSONTOLOGY, this.testOntologyInferredAxiomsUri1, false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
    }
    
    /**
     * This tests the behaviour when the given ontology URI does not infact exist in the
     * testOntology.rdf resource.
     * 
     * The desired behaviour is to add the version information for the designated ontology URI in
     * addition to the previous information, except for in cases where the previous information
     * referred to the same URI.
     * 
     * @throws RepositoryException
     * @throws RDFParseException
     * @throws IOException
     * @throws RDFHandlerException
     */
    @Test
    public void testStoreOntologyJustOntologyAndVersionUriFalse() throws RepositoryException, RDFParseException,
        IOException, RDFHandlerException
    {
        this.testOntology = new OntologyImpl();
        this.testOntology.setOntologyUri(this.testValueFactory.createURI("http://false.example.org/ontology/1"));
        this.testOntology.setOntologyVersionUri(this.testValueFactory
                .createURI("http://false.text.example.org/ontology/1/version/1.0"));
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        final Repository storeOntology =
                SesameUtils.storeOntology(this.testOntology, new InputStreamReader(testInputStream, "UTF-8"),
                        "http://false.base.example.org/test/1/", RDFFormat.RDFXML, this.testRepository,
                        SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        // we passed in a repository so we should get the same repository object out in the
        // SesameUtils implementation.
        // Note: This is not necessary for the DataHandler contract so do not do these tests in
        // DataHandler tests
        Assert.assertEquals(storeOntology, this.testRepository);
        
        // final URI managedOntologyVersionUri =
        this.testOntology.ensureVersionUriUniqueness(this.testRepository, false, true,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        Assert.assertNotNull(this.testOntology.getOntologyVersionUri());
        
        Assert.assertEquals(8, this.testRepositoryConnection.size());
        Assert.assertEquals(5, this.testRepositoryConnection.size(this.testOntology.getOntologyVersionUri()));
        Assert.assertEquals(3, this.testRepositoryConnection.size(SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OWL.VERSIONINFO, this.testValueFactory.createLiteral(this.testOntology.getOntologyVersionUri()
                        .stringValue(), OasConstants.XSD_ANYURI), false, this.testOntology.getOntologyVersionUri()));
        
        // Test that the actual information from the testOntology was imported
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(
                this.testValueFactory.createURI("http://test.example.org/ontology/0139"), RDF.TYPE, OWL.ONTOLOGY,
                false, this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(
                this.testValueFactory.createURI("http://test.example.org/ontology/0139"), OasConstants.OWL_VERSIONIRI,
                this.testValueFactory.createURI("http://test.example.org/ontology/0139/version:1"), false,
                this.testOntology.getOntologyVersionUri()));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OMV_CURRENTVERSION, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
    }
    
    @Test
    public void testStoreOntologyJustOntologyAndVersionUriTrue() throws RepositoryException, RDFParseException,
        IOException, RDFHandlerException
    {
        this.testOntology = new OntologyImpl();
        this.testOntology.setOntologyUri(this.testValueFactory.createURI("http://test.example.org/ontology/0139"));
        this.testOntology.setOntologyVersionUri(this.testValueFactory
                .createURI("http://test.example.org/ontology/0139/version:1"));
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        Assert.assertEquals(0, this.testRepositoryConnection.size());
        
        final Repository storeOntology =
                SesameUtils.storeOntology(this.testOntology, new InputStreamReader(testInputStream, "UTF-8"),
                        "http://test.base.example.org/test/1/", RDFFormat.RDFXML, this.testRepository,
                        SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        // we passed in a repository so we should get the same repository object out in the
        // SesameUtils implementation.
        // Note: This is not necessary for the DataHandler contract so do not do these tests in
        // DataHandler tests
        Assert.assertEquals(storeOntology, this.testRepository);
        
        // final URI managedOntologyVersionUri =
        this.testOntology.ensureVersionUriUniqueness(this.testRepository, false, true,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        Assert.assertNotNull(this.testOntology.getOntologyVersionUri());
        
        Assert.assertEquals(6, this.testRepositoryConnection.size());
        Assert.assertEquals(3, this.testRepositoryConnection.size(this.testOntology.getOntologyVersionUri()));
        Assert.assertEquals(3, this.testRepositoryConnection.size(SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OWL.VERSIONINFO, this.testValueFactory.createLiteral(this.testOntology.getOntologyVersionUri()
                        .stringValue(), OasConstants.XSD_ANYURI), false, this.testOntology.getOntologyVersionUri()));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OMV_CURRENTVERSION, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
    }
    
    /**
     * This test is aimed at the following parameters:
     * 
     * First ontology, both ontology URI and ontology version URI specified.
     * 
     * The input stream for the first ontology contains the same URIs as the ontology metadata, with
     * no extra metadata.
     * 
     * Second ontology, both ontology URI and a different ontology version URI specified.
     * 
     * The input stream for the second ontology contains one extra triple on top of the ontology
     * metadata that contains the same URIs as the in memory Ontology object.
     * 
     * @throws RepositoryException
     * @throws RDFParseException
     * @throws IOException
     * @throws RDFHandlerException
     */
    @Test
    public void testStoreOntologyJustOntologyAndVersionUriTrueTwice() throws RepositoryException, RDFParseException,
        IOException, RDFHandlerException
    {
        this.testOntology = new OntologyImpl();
        this.testOntology.setOntologyUri(this.testValueFactory.createURI("http://test.example.org/ontology/0139"));
        this.testOntology.setOntologyVersionUri(this.testValueFactory
                .createURI("http://test.example.org/ontology/0139/version:1"));
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        Assert.assertEquals(0, this.testRepositoryConnection.size());
        
        final Repository storeOntology =
                SesameUtils.storeOntology(this.testOntology, new InputStreamReader(testInputStream, "UTF-8"),
                        "http://test.base.example.org/test/1/", RDFFormat.RDFXML, this.testRepository,
                        SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        // we passed in a repository so we should get the same repository object out in the
        // SesameUtils implementation.
        // Note: This is not necessary for the DataHandler contract so do not do these tests in
        // DataHandler tests
        Assert.assertEquals(storeOntology, this.testRepository);
        
        // final URI managedOntologyVersionUri =
        this.testOntology.ensureVersionUriUniqueness(this.testRepository, false, true,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        final URI managedOntologyVersionUri = this.testOntology.getOntologyVersionUri();
        
        Assert.assertNotNull(managedOntologyVersionUri);
        
        Assert.assertEquals(6, this.testRepositoryConnection.size());
        Assert.assertEquals(3, this.testRepositoryConnection.size(managedOntologyVersionUri));
        Assert.assertEquals(3, this.testRepositoryConnection.size(SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, managedOntologyVersionUri));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, managedOntologyVersionUri, false, managedOntologyVersionUri));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OWL.VERSIONINFO, this.testValueFactory.createLiteral(this.testOntology.getOntologyVersionUri()
                        .stringValue(), OasConstants.XSD_ANYURI), false, managedOntologyVersionUri));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, managedOntologyVersionUri, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OMV_CURRENTVERSION, managedOntologyVersionUri, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        final InputStream testInputStream2 = this.getClass().getResourceAsStream("/testOntology1-2.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream2);
        
        final Ontology testOntology2 = new OntologyImpl();
        // same ontology URI as previous
        testOntology2.setOntologyUri(this.testOntology.getOntologyUri());
        testOntology2.setOntologyVersionUri(this.testValueFactory
                .createURI("http://test.example.org/ontology/0139/version:2"));
        
        final Repository storeOntology2 =
                SesameUtils.storeOntology(testOntology2, new InputStreamReader(testInputStream2, "UTF-8"),
                        "http://test.base.example.org/test/2/", RDFFormat.RDFXML, this.testRepository,
                        SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        // we passed in a repository so we should get the same repository object out in the
        // SesameUtils implementation.
        // Note: This is not necessary for the DataHandler contract so do not do these tests in
        // DataHandler tests
        Assert.assertEquals(storeOntology2, this.testRepository);
        
        // final URI managedOntologyVersionUri2 =
        testOntology2.ensureVersionUriUniqueness(this.testRepository, false, true, SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        final URI managedOntologyVersionUri2 = testOntology2.getOntologyVersionUri();
        
        Assert.assertNotNull(managedOntologyVersionUri2);
        
        Assert.assertEquals(13, this.testRepositoryConnection.size());
        Assert.assertEquals(3, this.testRepositoryConnection.size(managedOntologyVersionUri));
        Assert.assertEquals(5, this.testRepositoryConnection.size(managedOntologyVersionUri2));
        Assert.assertEquals(5, this.testRepositoryConnection.size(SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        // Verify that the original triples are still there
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, managedOntologyVersionUri));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, managedOntologyVersionUri, false, managedOntologyVersionUri));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OWL.VERSIONINFO, this.testValueFactory.createLiteral(this.testOntology.getOntologyVersionUri()
                        .stringValue(), OasConstants.XSD_ANYURI), false, managedOntologyVersionUri));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(testOntology2.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, managedOntologyVersionUri2));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(testOntology2.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, managedOntologyVersionUri2, false, managedOntologyVersionUri2));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(testOntology2.getOntologyUri(), OWL.VERSIONINFO,
                this.testValueFactory.createLiteral(testOntology2.getOntologyVersionUri().stringValue(),
                        OasConstants.XSD_ANYURI), false, managedOntologyVersionUri2));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OWL.PRIORVERSION, managedOntologyVersionUri, false, managedOntologyVersionUri2));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(testOntology2.getOntologyUri(), RDFS.LABEL,
                this.testValueFactory.createLiteral("Label for version 2 of test ontology 0139"), false,
                managedOntologyVersionUri2));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OMV_CURRENTVERSION, managedOntologyVersionUri2, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, managedOntologyVersionUri2, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, managedOntologyVersionUri, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OWL.PRIORVERSION, managedOntologyVersionUri, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        // LOG.error("ontology manager graph="+this.testRepositoryConnection.getStatements(null,
        // null, null, false, UriConstants.ONTOLOGY_MANAGER_GRAPH).asList());
    }
    
    /**
     * This tests the behaviour when the given ontology URI does not infact exist in the
     * testOntology.rdf resource.
     * 
     * The desired behaviour is to add the version information for the designated ontology URI in
     * addition to the previous information, except for in cases where the previous information
     * referred to the same URI.
     * 
     * @throws RepositoryException
     * @throws RDFParseException
     * @throws IOException
     * @throws RDFHandlerException
     */
    @Test
    public void testStoreOntologyJustOntologyUriFalse() throws RepositoryException, RDFParseException, IOException,
        RDFHandlerException
    {
        this.testOntology = new OntologyImpl();
        this.testOntology.setOntologyUri(this.testValueFactory.createURI("http://false.example.org/ontology/1"));
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        final Repository storeOntology =
                SesameUtils.storeOntology(this.testOntology, new InputStreamReader(testInputStream, "UTF-8"),
                        "http://false.base.example.org/test/1/", RDFFormat.RDFXML, this.testRepository,
                        SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        // we passed in a repository so we should get the same repository object out in the
        // SesameUtils implementation.
        // Note: This is not necessary for the DataHandler contract so do not do these tests in
        // DataHandler tests
        Assert.assertEquals(storeOntology, this.testRepository);
        
        // final URI managedOntologyVersionUri =
        // this.testOntology
        // .getManagedOntologyVersionUri(this.testRepository, UriConstants.ONTOLOGY_MANAGER_GRAPH);
        
        Assert.assertEquals(8, this.testRepositoryConnection.size());
        Assert.assertEquals(5, this.testRepositoryConnection.size(this.testOntology.getOntologyVersionUri()));
        Assert.assertEquals(3, this.testRepositoryConnection.size(SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                this.testOntology.getOntologyVersionUri()));
        
        // Test that the actual information from the testOntology was imported
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(
                this.testValueFactory.createURI("http://test.example.org/ontology/0139"), RDF.TYPE, OWL.ONTOLOGY,
                false, this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(
                this.testValueFactory.createURI("http://test.example.org/ontology/0139"), OasConstants.OWL_VERSIONIRI,
                this.testValueFactory.createURI("http://test.example.org/ontology/0139/version:1"), false,
                this.testOntology.getOntologyVersionUri()));
        // Assert.assertTrue(this.testRepositoryConnection.hasStatement(
        // this.testValueFactory.createURI("http://test.example.org/ontology/0139"),
        // UriConstants.OWL_VERSIONIRI,
        // this.testValueFactory.createLiteral("", XSDVocabulary.ANY_URI.getIRI().toOpenRDFURI()),
        // false,
        // this.testOntology.getOntologyVersionUri()));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OMV_CURRENTVERSION, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        // FIXME: Determine what the 5th statement in the ontology version URI graph is
        
    }
    
    @Test
    public void testStoreOntologyJustOntologyUriTrue() throws RepositoryException, RDFParseException, IOException,
        RDFHandlerException
    {
        this.testOntology = new OntologyImpl();
        this.testOntology.setOntologyUri(this.testValueFactory.createURI("http://test.example.org/ontology/0139"));
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        Assert.assertEquals(0, this.testRepositoryConnection.size());
        
        final Repository storeOntology =
                SesameUtils.storeOntology(this.testOntology, new InputStreamReader(testInputStream, "UTF-8"),
                        "http://test.base.example.org/test/1/", RDFFormat.RDFXML, this.testRepository,
                        SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        // we passed in a repository so we should get the same repository object out in the
        // SesameUtils implementation.
        // Note: This is not necessary for the DataHandler contract so do not do these tests in
        // DataHandler tests
        Assert.assertEquals(storeOntology, this.testRepository);
        
        // final URI managedOntologyVersionUri =
        this.testOntology.ensureVersionUriUniqueness(this.testRepository, false, true,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        Assert.assertNotNull(this.testOntology.getOntologyVersionUri());
        
        Assert.assertEquals(6, this.testRepositoryConnection.size());
        Assert.assertEquals(3, this.testRepositoryConnection.size(this.testOntology.getOntologyVersionUri()));
        Assert.assertEquals(3, this.testRepositoryConnection.size(SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                this.testOntology.getOntologyVersionUri()));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OMV_CURRENTVERSION, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
    }
    
    @Test
    public void testStoreOntologyMediumSize() throws RepositoryException, RDFParseException, IOException,
        RDFHandlerException
    {
        this.testOntology = new OntologyImpl();
        this.testOntology.setOntologyUri(this.testValueFactory.createURI("http://purl.obolibrary.org/obo/po.owl"));
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        final Repository storeOntology =
                SesameUtils.storeOntology(this.testOntology, new InputStreamReader(testInputStream, "UTF-8"),
                        "http://test.base.example.org/test/1/", RDFFormat.RDFXML, this.testRepository,
                        SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        // we passed in a repository so we should get the same repository object out in the
        // SesameUtils implementation.
        // Note: This is not necessary for the DataHandler contract so do not do these tests in
        // DataHandler tests
        Assert.assertEquals(storeOntology, this.testRepository);
        
        // final URI managedOntologyVersionUri =
        this.testOntology.ensureVersionUriUniqueness(this.testRepository, false, true,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        Assert.assertNotNull(this.testOntology.getOntologyVersionUri());
        
        Assert.assertEquals(44290, this.testRepositoryConnection.size());
        // Original ontology is 44285, plus one triple for the owl:versionIRI, as the ontology only
        // contains owl:versionInfo
        Assert.assertEquals(44287, this.testRepositoryConnection.size(this.testOntology.getOntologyVersionUri()));
        Assert.assertEquals(3, this.testRepositoryConnection.size(SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
    }
    
    @Test
    public void testStoreOntologyNullOntology() throws RepositoryException, RDFParseException, IOException,
        RDFHandlerException
    {
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        try
        {
            SesameUtils.storeOntology((Ontology)null, new InputStreamReader(testInputStream, "UTF-8"),
                    "http://test.base.example.org/test/1/", RDFFormat.RDFXML, this.testRepository,
                    SesameUtils.ONTOLOGY_MANAGER_GRAPH);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final IllegalArgumentException iae)
        {
            Assert.assertTrue(iae.getMessage().contains("Ontology must not be null"));
        }
    }
    
    @Test
    public void testStoreOntologyOnlyOntologyVersionUri() throws RepositoryException, RDFParseException, IOException,
        RDFHandlerException
    {
        this.testOntology = new OntologyImpl();
        this.testOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        try
        {
            SesameUtils.storeOntology(this.testOntology, new InputStreamReader(testInputStream, "UTF-8"),
                    "http://test.base.example.org/test/1/", RDFFormat.RDFXML, this.testRepository,
                    SesameUtils.ONTOLOGY_MANAGER_GRAPH);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final IllegalArgumentException iae)
        {
            Assert.assertTrue(iae.getMessage().contains("Ontology URI must not be null"));
        }
        
    }
    
    @Test
    public void testStoreOntologyProfilesAll() throws RepositoryException, RDFParseException, IOException,
        RDFHandlerException
    {
        // Check that the profile registry was not empty
        // TODO: Update this whenever the list of standard profiles changes or we register a custom
        // profile
        Assert.assertEquals(5, this.testAllProfiles.size());
        
        this.testOntology = new OntologyImpl();
        this.testOntology.setOntologyUri(this.testValueFactory.createURI("http://test.example.org/ontology/0139"));
        this.testOntology.setOntologyVersionUri(this.testValueFactory
                .createURI("http://test.example.org/ontology/0139/version:1"));
        this.testOntology.setProfiles(this.testAllProfiles);
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        Assert.assertEquals(0, this.testRepositoryConnection.size());
        
        final StatementCollector statementCollector = new StatementCollector();
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFXML);
        parser.setRDFHandler(statementCollector);
        parser.parse(new InputStreamReader(testInputStream, "UTF-8"), "http://test.base.example.org/test/1/");
        
        final ArrayList<Statement> nonEmptyInferredStatements = new ArrayList<Statement>(1);
        nonEmptyInferredStatements.add(this.testValueFactory.createStatement(this.testOntologyInferredAxiomsUri1,
                RDF.TYPE, OWL.ONTOLOGY));
        
        final Repository storeOntology =
                SesameUtils.storeOntology(this.testOntology, statementCollector.getStatements(),
                        statementCollector.getNamespaces(), nonEmptyInferredStatements, this.testRepository,
                        SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        // we passed in a repository so we should get the same repository object out in the
        // SesameUtils implementation.
        // Note: This is not necessary for the DataHandler contract so do not do these tests in
        // DataHandler tests
        Assert.assertEquals(storeOntology, this.testRepository);
        
        // final URI managedOntologyVersionUri =
        this.testOntology.ensureVersionUriUniqueness(this.testRepository, false, true,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        Assert.assertNotNull(this.testOntology.getOntologyVersionUri());
        
        // TODO: Update this number if the number of profiles changes
        Assert.assertEquals(11, this.testRepositoryConnection.size());
        Assert.assertEquals(3, this.testRepositoryConnection.size(this.testOntology.getOntologyVersionUri()));
        // TODO: Update this number if the number of profiles changes
        Assert.assertEquals(8, this.testRepositoryConnection.size(SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OWL.VERSIONINFO, this.testValueFactory.createLiteral(this.testOntology.getOntologyVersionUri()
                        .stringValue(), OasConstants.XSD_ANYURI), false, this.testOntology.getOntologyVersionUri()));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OMV_CURRENTVERSION, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        for(final OWLProfile nextProfile : this.testAllProfiles)
        {
            Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyVersionUri(),
                    OasConstants.OAS_INPROFILE, nextProfile.getIRI().toOpenRDFURI(), false,
                    SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        }
    }
    
    @Test
    public void testStoreOntologyProfilesEmpty() throws RepositoryException, RDFParseException, IOException,
        RDFHandlerException
    {
        Assert.assertTrue(this.testEmptyProfiles.isEmpty());
        
        this.testOntology = new OntologyImpl();
        this.testOntology.setOntologyUri(this.testValueFactory.createURI("http://test.example.org/ontology/0139"));
        this.testOntology.setOntologyVersionUri(this.testValueFactory
                .createURI("http://test.example.org/ontology/0139/version:1"));
        this.testOntology.setProfiles(this.testEmptyProfiles);
        
        final InputStream testInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Test file not found", testInputStream);
        
        Assert.assertEquals(0, this.testRepositoryConnection.size());
        
        final StatementCollector statementCollector = new StatementCollector();
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFXML);
        parser.setRDFHandler(statementCollector);
        parser.parse(new InputStreamReader(testInputStream, "UTF-8"), "http://test.base.example.org/test/1/");
        
        final ArrayList<Statement> nonEmptyInferredStatements = new ArrayList<Statement>(1);
        nonEmptyInferredStatements.add(this.testValueFactory.createStatement(this.testOntologyInferredAxiomsUri1,
                RDF.TYPE, OWL.ONTOLOGY));
        
        final Repository storeOntology =
                SesameUtils.storeOntology(this.testOntology, statementCollector.getStatements(),
                        statementCollector.getNamespaces(), nonEmptyInferredStatements, this.testRepository,
                        SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        // we passed in a repository so we should get the same repository object out in the
        // SesameUtils implementation.
        // Note: This is not necessary for the DataHandler contract so do not do these tests in
        // DataHandler tests
        Assert.assertEquals(storeOntology, this.testRepository);
        
        // final URI managedOntologyVersionUri =
        this.testOntology.ensureVersionUriUniqueness(this.testRepository, false, true,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH);
        
        Assert.assertNotNull(this.testOntology.getOntologyVersionUri());
        
        Assert.assertEquals(6, this.testRepositoryConnection.size());
        Assert.assertEquals(3, this.testRepositoryConnection.size(this.testOntology.getOntologyVersionUri()));
        Assert.assertEquals(3, this.testRepositoryConnection.size(SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                this.testOntology.getOntologyVersionUri()));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OWL.VERSIONINFO, this.testValueFactory.createLiteral(this.testOntology.getOntologyVersionUri()
                        .stringValue(), OasConstants.XSD_ANYURI), false, this.testOntology.getOntologyVersionUri()));
        
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(), RDF.TYPE,
                OWL.ONTOLOGY, false, SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OWL_VERSIONIRI, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
        Assert.assertTrue(this.testRepositoryConnection.hasStatement(this.testOntology.getOntologyUri(),
                OasConstants.OMV_CURRENTVERSION, this.testOntology.getOntologyVersionUri(), false,
                SesameUtils.ONTOLOGY_MANAGER_GRAPH));
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.utils.SesameUtils#toRDFRepository(com.github.ansell.oas.objects.Annotation, org.openrdf.repository.Repository, URI)}
     * .
     */
    @Test
    public final void testToRDFRepositoryAnnotationRepository() throws Exception
    {
        this.testAnnotation = new AnnotationImpl();
        
        this.testAnnotation.setAnnotationUniqueId(this.testAnnotationUri1);
        this.testAnnotation.setAnnotatedObjectUri(this.testAnnotationObjectUri1);
        this.testAnnotation.setAnnotatedObjectType(this.testAnnotationObjectTypeUri1);
        this.testAnnotation.setAnnotator(this.testAnnotationAnnotatorUri1);
        this.testAnnotation.setOntologyTermUri(this.testAnnotationOntologyTermUri1);
        this.testAnnotation.setTag(this.testAnnotationTagLiteral1);
        
        Assert.assertEquals(0, this.testRepositoryConnection.size());
        
        SesameUtils.toRDFRepository(this.testAnnotation, this.testRepository, SesameUtils.ANNOTATION_MANAGEMENT_GRAPH);
        
        Assert.assertEquals(7, this.testRepositoryConnection.size());
        
        // DebugUtils.printContexts(this.testRepositoryConnection);
        
        Assert.assertTrue("Repository connection did not contain the rdf:type statement for the annotation object",
                this.testRepositoryConnection.hasStatement(this.testAnnotationUri1, RDF.TYPE,
                        OasConstants.TAGGING_TAGGING, true, SesameUtils.ANNOTATION_MANAGEMENT_GRAPH));
        Assert.assertTrue("Repository connection did not contain the rdf:type statement for the annotation object",
                this.testRepositoryConnection.hasStatement(this.testAnnotationUri1, RDF.TYPE,
                        OasConstants.TAGGING_TAGGING, true, this.testAnnotationUri1));
        Assert.assertTrue(
                "Repository connection did not contain the tagging:taggedResource statement for the annotation object",
                this.testRepositoryConnection.hasStatement(this.testAnnotationUri1,
                        OasConstants.TAGGING_TAGGED_RESOURCE, this.testAnnotationObjectUri1, true,
                        this.testAnnotationUri1));
        Assert.assertTrue("Repository connection did not contain the foaf:maker statement for the annotation object",
                this.testRepositoryConnection.hasStatement(this.testAnnotationUri1, OasConstants.FOAF_MAKER,
                        this.testAnnotationAnnotatorUri1, true, this.testAnnotationUri1));
        Assert.assertTrue(
                "Repository connection did not contain the moat:tagMeaning statement for the annotation object",
                this.testRepositoryConnection.hasStatement(this.testAnnotationUri1, OasConstants.MOAT_TAG_MEANING,
                        this.testAnnotationOntologyTermUri1, true, this.testAnnotationUri1));
        Assert.assertTrue("Repository connection did not contain the tagging:name statement for the annotation object",
                this.testRepositoryConnection.hasStatement(this.testAnnotationUri1, OasConstants.TAGGING_NAME,
                        this.testAnnotationTagLiteral1, true, this.testAnnotationUri1));
        Assert.assertTrue("Repository connection did not contain the rdf:type statement for the annotated object",
                this.testRepositoryConnection.hasStatement(this.testAnnotationObjectUri1, RDF.TYPE,
                        this.testAnnotationObjectTypeUri1, true, this.testAnnotationUri1));
    }
    
    @Test
    public final void testToRdfRepositoryOntologyComplete() throws RepositoryException
    {
        this.testOntology = new OntologyImpl();
        
        this.testOntology.setOntologyUri(this.testOntologyUri1);
        this.testOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final Repository rdfRepository = SesameUtils.toRDFRepository(this.testOntology, this.testRepository);
        
        RepositoryConnection conn = null;
        
        try
        {
            conn = rdfRepository.getConnection();
            
            Assert.assertEquals(2, conn.size());
            
            this.testOntology.ensureVersionUriUniqueness(this.testRepository, true, false,
                    SesameUtils.ONTOLOGY_MANAGER_GRAPH);
            
            Assert.assertEquals(2, conn.size(this.testOntology.getOntologyVersionUri()));
        }
        finally
        {
            if(conn != null)
            {
                conn.close();
            }
        }
    }
    
    @Test
    public final void testToRdfRepositoryOntologyJustOntologyUri() throws RepositoryException
    {
        this.testOntology = new OntologyImpl();
        
        this.testOntology.setOntologyUri(this.testOntologyUri1);
        
        final Repository rdfRepository = SesameUtils.toRDFRepository(this.testOntology, this.testRepository);
        
        RepositoryConnection conn = null;
        
        try
        {
            conn = rdfRepository.getConnection();
            
            Assert.assertEquals(2, conn.size());
            
            // URI managedOntologyVersionUri =
            this.testOntology.ensureVersionUriUniqueness(this.testRepository, true, false,
                    SesameUtils.ONTOLOGY_MANAGER_GRAPH);
            
            Assert.assertEquals(2, conn.size());
            Assert.assertEquals(2, conn.size(this.testOntology.getOntologyVersionUri()));
        }
        finally
        {
            if(conn != null)
            {
                conn.close();
            }
        }
    }
    
    @Test
    public final void testToRdfRepositoryOntologyJustOntologyVersionUri() throws RepositoryException
    {
        this.testOntology = new OntologyImpl();
        
        this.testOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        try
        {
            SesameUtils.toRDFRepository(this.testOntology, this.testRepository);
            
            Assert.fail("Did not find expected exception");
        }
        catch(final IllegalArgumentException iae)
        {
            Assert.assertTrue(iae.getMessage().contains("Every ontology must have a URI"));
        }
        
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.utils.SesameUtils#toRDFRepository(com.github.ansell.oas.objects.ResourceCount, org.openrdf.repository.Repository)}
     * .
     */
    @Test
    public final void testToRDFRepositoryResourceCountRepository()
    {
        this.testResourceCount = new UriCountImpl();
        this.testResourceCount.setCount(73);
        
        try
        {
            Assert.assertEquals(0, this.testRepositoryConnection.size());
            
            SesameUtils.toRDFRepository(this.testResourceCount, this.testRepository);
            
            Assert.assertEquals(2, this.testRepositoryConnection.size());
            
            // TODO: should we support URIs for counts explicitly
            Assert.assertTrue(
                    "Repository connection did not contain the rdf:type statement for the resource count object",
                    this.testRepositoryConnection.hasStatement(null, RDF.TYPE, OasConstants.URN_TAG_OBJECT_URI_COUNT,
                            true));
            Assert.assertTrue(
                    "Repository connection did not contain the tag:uriCount statement for the resource count object",
                    this.testRepositoryConnection.hasStatement(null, OasConstants.OAS_URI_DIRECT_COUNT,
                            this.testValueFactory.createLiteral(73), true));
        }
        catch(final RepositoryException ex)
        {
            Assert.fail("Found unexpected RepositoryException");
        }
        
    }
    
}
