package com.github.ansell.oas.webservice.impl.test;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.io.IOUtils;
import org.junit.Assert;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.vocabulary.OWL;
import org.openrdf.model.vocabulary.RDF;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.RDFParseException;
import org.openrdf.rio.RDFParser;
import org.openrdf.rio.Rio;
import org.openrdf.rio.helpers.StatementCollector;
import org.restlet.data.MediaType;
import org.restlet.data.Preference;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;

import com.github.ansell.oas.utils.OasConstants;
import com.github.ansell.oas.utils.OasProps;

public class AbstractOntologyManagerTest extends AbstractResourceImplTest
{
    
    public AbstractOntologyManagerTest()
    {
        super();
    }
    
    /**
     * A utility method to check the current state of the ontology manager as displayed by the
     * default index page that is displayed when no ontologyUri is given in the query.
     * 
     * @param mediaType
     *            The MediaType to use when calling the ontology manager. Must be an RDF compatible
     *            MediaType.
     * @param rdfFormat
     *            The RDFFormat to use when parsing the ontology manager index. Must have an
     *            accessible RDFParser instance.
     * @param expectedNumberOfOntologies
     *            The exact number of unique ontology URIs to expect.
     * @param expectedNumberOfVersionIRIs
     *            The exact number of unique ontology version URIs to expect.
     * @param partialOntologyUriList
     *            A partial list of ontology URIs that must be displayed.
     * @param partialOntologyVersionIRIList
     *            A partial list of ontology version URIs that must be displayed.
     * @return A Collection of Statement objects that were parsed from the page. May contain
     *         duplicates.
     * @throws IOException
     * @throws RDFParseException
     * @throws RDFHandlerException
     */
    protected Collection<Statement> checkOntologyManager(final MediaType mediaType, final RDFFormat rdfFormat,
            final int expectedNumberOfOntologies, final int expectedNumberOfVersionIRIs,
            final Collection<URI> partialOntologyUriList, final Collection<URI> partialOntologyVersionIRIList)
        throws IOException, RDFParseException, RDFHandlerException
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        final List<Preference<MediaType>> connNegList = new ArrayList<Preference<MediaType>>(1);
        connNegList.add(new Preference<MediaType>(mediaType, 1.0f));
        
        creationClientResource.getClientInfo().setAcceptedMediaTypes(connNegList);
        
        final Representation results = creationClientResource.get();
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, creationClientResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(mediaType, results.getMediaType());
        
        final String resultString = IOUtils.toString(results.getStream());
        
        final RDFParser parser = Rio.createParser(rdfFormat);
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new StringReader(resultString), this.getUrl(this.propertyUtil.get(
                OasProps.PROP_ONTOLOGY_MANAGER_PATH, OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
        
        if(this.log.isDebugEnabled())
        {
            for(final Statement nextCollectedStatement : statementCollector.getStatements())
            {
                this.log.debug("nextCollectedStatement: {}", nextCollectedStatement);
            }
        }
        
        // Assert.assertEquals(expectedNumberOfOntologies,
        // statementCollector.getStatements().size());
        
        for(final URI nextOntologyUri : partialOntologyUriList)
        {
            boolean foundUri = false;
            for(final Statement nextStatement : statementCollector.getStatements())
            {
                if(nextStatement.getSubject().equals(nextOntologyUri) && nextStatement.getPredicate().equals(RDF.TYPE)
                        && nextStatement.getObject().equals(OWL.ONTOLOGY))
                {
                    foundUri = true;
                    break;
                }
            }
            
            Assert.assertTrue("Did not find expected ontologyUri: " + nextOntologyUri.stringValue(), foundUri);
        }
        
        for(final URI nextOntologyVersionUri : partialOntologyVersionIRIList)
        {
            boolean foundUri = false;
            for(final Statement nextStatement : statementCollector.getStatements())
            {
                if(nextStatement.getPredicate().equals(OasConstants.OWL_VERSIONIRI)
                        && nextStatement.getObject().equals(nextOntologyVersionUri))
                {
                    foundUri = true;
                    break;
                }
            }
            
            Assert.assertTrue("Did not find expected ontologyVersionIri: " + nextOntologyVersionUri.stringValue(),
                    foundUri);
        }
        
        final Set<URI> ontologyUris = new HashSet<URI>();
        final Set<URI> ontologyVersionUris = new HashSet<URI>();
        
        for(final Statement nextStatement : statementCollector.getStatements())
        {
            if(nextStatement.getPredicate().equals(RDF.TYPE) && nextStatement.getObject().equals(OWL.ONTOLOGY))
            {
                if(!ontologyUris.contains(nextStatement.getSubject()))
                {
                    ontologyUris.add((URI)nextStatement.getSubject());
                }
            }
            else if(nextStatement.getPredicate().equals(OasConstants.OWL_VERSIONIRI))
            {
                if(!ontologyVersionUris.contains(nextStatement.getObject()))
                {
                    ontologyVersionUris.add((URI)nextStatement.getObject());
                }
            }
            else
            {
                this.log.debug("Found unknown statement in ontologyManager RDF/XML: {}", nextStatement);
            }
        }
        
        Assert.assertEquals("Did not find expected number of ontologies", expectedNumberOfOntologies,
                ontologyUris.size());
        
        Assert.assertEquals("Did not find expected number of ontology version IRIs", expectedNumberOfVersionIRIs,
                ontologyVersionUris.size());
        
        return statementCollector.getStatements();
    }
    
}