package com.github.ansell.oas.webservice.impl.test;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.util.UUID;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Rule;
import org.junit.rules.Timeout;
import org.openrdf.model.Model;
import org.openrdf.model.impl.LinkedHashModel;
import org.openrdf.repository.RepositoryException;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.RDFHandlerException;
import org.openrdf.rio.RDFParseException;
import org.openrdf.rio.RDFParser;
import org.openrdf.rio.Rio;
import org.openrdf.rio.helpers.StatementCollector;
import org.restlet.Component;
import org.restlet.data.CharacterSet;
import org.restlet.data.Language;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Protocol;
import org.restlet.data.Status;
import org.restlet.ext.html.FormData;
import org.restlet.ext.html.FormDataSet;
import org.restlet.representation.AppendableRepresentation;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.bridge.SLF4JBridgeHandler;

import com.github.ansell.oas.objects.Annotation;
import com.github.ansell.oas.test.TestUtils;
import com.github.ansell.oas.utils.ApplicationUtils;
import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.oas.webservice.OasWebServiceApplication;
import com.github.ansell.oas.webservice.impl.OasWebServiceApplicationImpl;
import com.github.ansell.propertyutil.PropertyUtil;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * Abstract test implementation that contains common components required by resource implementation
 * tests, including setting up the application and component, along with the TEST_PORT number to use
 * in the tests.
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public class AbstractResourceImplTest
{
    static
    {
        // add the ability to view java.util.logging messages using SLF4J
        System.getProperties().put("org.restlet.engine.loggerFacadeClass", "org.restlet.ext.slf4j.Slf4jLoggerFacade");
        SLF4JBridgeHandler.install();
    }
    
    /**
     * Timeout tests after 1 minute.
     */
    @Rule
    public Timeout timeout = new Timeout(60000);
    
    /**
     * Determines the TEST_PORT number to use for the test server
     */
    protected static final int TEST_PORT = 8182;
    
    protected final Logger log = LoggerFactory.getLogger(this.getClass());
    
    /**
     * A constant used to make requests that require admin privileges easier to recognise inside
     * tests.
     */
    protected final boolean testWithAdminPrivileges = true;
    
    /**
     * A constant used to make requests that do not require admin privileges easier to recognise
     * inside tests.
     */
    protected final boolean testNoAdminPrivileges = false;
    
    private Component component;
    
    protected PropertyUtil propertyUtil;
    
    public AbstractResourceImplTest()
    {
        super();
    }
    
    /**
     * Utility method to verify that freemarker has not encountered errors when generating output.
     * 
     * @param body
     *            Generated output in which to look for possible freemarker errors.
     */
    public void assertFreemarker(final String body)
    {
        Assert.assertFalse("Freemarker error.", body.contains("Java backtrace for programmers"));
        Assert.assertFalse("Freemarker error.", body.contains("freemarker.core"));
        Assert.assertFalse("Freemarker error.", body.contains("Error: Page generation failed"));
    }
    
    /**
     * Parses the given input, asserts that the number of statements is as expected, and returns a
     * model containing all of the parsed statements.
     * 
     * @param input
     *            The {@link InputStream} containing the RDF document.
     * @param expectedFormat
     *            The {@link RDFFormat} that corresponds to the input stream.
     * @param expectedNumberOfStatements
     *            The expected total number of statements to be created from the input stream. Note,
     *            as we are using {@link Model} internally this means that quads formats may
     *            generate duplicate triples that have different contexts, and each of the contexts
     *            will effectively add another triple to the expected number of statements.
     * @return A {@link Model} containing all of the parsed statements.
     * @throws RDFParseException
     * @throws RDFHandlerException
     * @throws IOException
     * @throws RepositoryException
     */
    protected Model assertRdf(final InputStream input, final RDFFormat expectedFormat,
            final int expectedNumberOfStatements) throws RDFParseException, RDFHandlerException, IOException
    {
        final Model graph = new LinkedHashModel();
        final RDFParser parser = Rio.createParser(expectedFormat);
        parser.setRDFHandler(new StatementCollector(graph));
        parser.parse(input, "");
        
        // DebugUtils.printStatements(graph);
        // DebugUtils.printContexts(graph);
        // DebugUtils.printStatements(graph, null);
        // DebugUtils.printStatements(graph,
        // ValueFactoryImpl.getInstance().createURI("urn:tag:oas:annotations:managementgraph"));
        Assert.assertEquals("Did not find expected number of statements", expectedNumberOfStatements, graph.size());
        
        return graph;
    }
    
    /**
     * Parses the given input, asserts that the number of statements is as expected, and returns a
     * model containing all of the parsed statements.
     * 
     * @param input
     *            The {@link Reader} containing the RDF document.
     * @param expectedFormat
     *            The {@link RDFFormat} that corresponds to the input stream.
     * @param expectedNumberOfStatements
     *            The expected total number of statements to be created from the input stream. Note,
     *            as we are using {@link Model} internally this means that quads formats may
     *            generate duplicate triples that have different contexts, and each of the contexts
     *            will effectively add another triple to the expected number of statements.
     * @return A {@link Model} containing all of the parsed statements.
     * @throws RDFParseException
     * @throws RDFHandlerException
     * @throws IOException
     * @throws RepositoryException
     */
    protected Model assertRdf(final Reader input, final RDFFormat expectedFormat, final int expectedNumberOfStatements)
        throws RDFParseException, RDFHandlerException, IOException
    {
        final Model graph = new LinkedHashModel();
        final RDFParser parser = Rio.createParser(expectedFormat);
        parser.setRDFHandler(new StatementCollector(graph));
        parser.parse(input, "");
        
        // DebugUtils.printStatements(graph);
        // DebugUtils.printContexts(graph);
        // DebugUtils.printStatements(graph, null);
        // DebugUtils.printStatements(graph,
        // ValueFactoryImpl.getInstance().createURI("urn:tag:oas:annotations:managementgraph"));
        Assert.assertEquals("Did not find expected number of statements", expectedNumberOfStatements, graph.size());
        
        return graph;
    }
    
    protected void createTestAnnotations(final int numberOfAnnotationsToCreate) throws Exception
    {
        for(int i = 0; i < numberOfAnnotationsToCreate; i++)
        {
            // Get the resource and dump it to the console
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                            OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
            
            final String taggedResourceUri =
                    "http://example.org/podd/sample/" + i + ":uuid:" + UUID.randomUUID().toString();
            final String authorUri = "http://example.org/peter" + i;
            final String tagName = "leaf measurement" + i;
            final String ontologyUri = "http://purl.org/obo/owl/PO#PO_0006016" + i;
            final String taggedResourceType = "http://www.podd.org/poddModel#Measurement" + i;
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String rdfDocument =
                    TestUtils.createTestAnnotationDocument(MediaType.APPLICATION_RDF_XML.getName(), null,
                            taggedResourceUri, taggedResourceType, ontologyUri, tagName, authorUri);
            
            final Representation input =
                    new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            final Representation createRepresentation =
                    RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        }
    }
    
    /**
     * Create a new user with username "testNewUser".
     * 
     * @return True if the user creation was successful.
     * @throws Exception
     */
    protected void createTestUser() throws Exception
    {
        this.createTestUser("testNewUser", "testNewPassword", "test@email.com", "Usersmith", "Mister");
    }
    
    /**
     * Create a new user with the given username, password, email, lastname and firstname.
     * 
     * @param username
     * @param password
     * @param email
     * @param lastName
     * @param firstName
     * @return True if the user creation was successful.
     * @throws Exception
     */
    protected void createTestUser(final String username, final String password, final String email,
            final String lastName, final String firstName) throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_CREATE_USER_PATH,
                        OasProps.DEF_CREATE_USER_PATH)));
        
        final FormDataSet form = new FormDataSet();
        form.setMultipart(true);
        form.getEntries().add(new FormData("username", username));
        form.getEntries().add(new FormData("password", password));
        form.getEntries().add(new FormData("email", email));
        form.getEntries().add(new FormData("lastName", lastName));
        form.getEntries().add(new FormData("firstName", firstName));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, form,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final ClientResource fetchClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_GET_USER_PATH,
                        OasProps.DEF_GET_USER_PATH)));
        fetchClientResource.getReference().addQueryParameter("username", "testNewUser");
        
        final Representation fetchResults =
                RestletTestUtils.doTestAuthenticatedRequest(fetchClientResource, Method.GET, null, MediaType.TEXT_HTML,
                        Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        Assert.assertNotNull("Did not successfully create test user", fetchResults);
    }
    
    /**
     * @param parsedResultAnnotation
     * @return
     */
    protected String getAnnotationUrl(final Annotation parsedResultAnnotation)
    {
        return this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_PATH, OasProps.DEF_ANNOTATION_PATH) + "?"
                + OasProps.PARAM_ANNOTATIONID + "=" + parsedResultAnnotation.getAnnotationUniqueId().stringValue());
    }
    
    /**
     * Returns the URI that can be used to access the given path.
     * 
     * @param path
     *            The path on the temporary test server to access. This path must start with a
     *            slash.
     * @return A full URI that can be used to dereference the given path on the test server.
     */
    public String getUrl(final String path)
    {
        if(!path.startsWith("/"))
        {
            return "http://localhost:" + AbstractResourceImplTest.TEST_PORT + "/oas-test/" + path;
        }
        else
        {
            return "http://localhost:" + AbstractResourceImplTest.TEST_PORT + "/oas-test" + path;
        }
    }
    
    /**
     * Create a new server for each test.
     * 
     * State will only be shared when they use a common database.
     * 
     * @throws Exception
     */
    @Before
    public void setUp() throws Exception
    {
        this.component = new Component();
        
        // Add a new HTTP server listening on the given TEST_PORT.
        this.component.getServers().add(Protocol.HTTP, AbstractResourceImplTest.TEST_PORT);
        
        this.component.getClients().add(Protocol.CLAP);
        this.component.getClients().add(Protocol.HTTP);
        
        final OasWebServiceApplication nextApplication =
                new OasWebServiceApplicationImpl(this.component.getContext().createChildContext());
        
        // Setup a new dummy data handler for each test to ensure they are segregated
        // NOTE: the configuration for the repository to be used is derived
        // from oas.sesame.url in src/test/resources/oas.properties
        // If oas.sesame.url is not defined, or it is an empty string, a new memory store
        // will be created for each instance of OasWebServiceApplication,
        // which in effect means that it will be recreated for each test
        // application.setDataHandler(new SesameDataHandlerImpl());
        
        // Attach the sample application.
        this.component.getDefaultHost().attach("/oas-test/", nextApplication);
        
        // The application cannot be setup properly until it is attached, as it requires
        // Application.getContext() to not return null
        ApplicationUtils.setupApplication(nextApplication, nextApplication.getContext());
        
        // Start the component.
        this.component.start();
        
        this.propertyUtil = nextApplication.getPropertyUtil();
    }
    
    /**
     * Stop and nullify the test server object after each test.
     * 
     * NOTE: Does not clear any databases.
     * 
     * @throws Exception
     */
    @After
    public void tearDown() throws Exception
    {
        // Stop the component
        if(this.component != null)
        {
            this.component.stop();
        }
        
        // nullify the reference to the component
        this.component = null;
    }
    
}