/**
 * OAS Webservice test implementations
 */
package com.github.ansell.oas.webservice.impl.test;

import java.util.Map;

import org.junit.Assert;
import org.junit.Test;
import org.openrdf.model.URI;
import org.restlet.data.CharacterSet;
import org.restlet.data.Language;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.representation.AppendableRepresentation;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;

import com.github.ansell.oas.objects.Annotation;
import com.github.ansell.oas.test.TestUtils;
import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.oas.utils.SesameUtils;
import com.github.ansell.restletutils.RestletUtilMediaType;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * Unit test for AnnotationByIdResourceImpl.
 */
public class AnnotationByIdResourceImplTest extends AbstractResourceImplTest
{
    /**
     * Test whether Annotations can be stored and retrieved using the application with an RDF/XML
     * request and an HTML response.
     */
    @Test
    public void testAnnotationCreationJson2Json() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        final String taggedResourceUri = "http://example.org/podd/sample/1";
        final String authorUri = "http://example.org/peter";
        final String tagName = "leaf measurement";
        final String ontologyUri = "http://purl.org/obo/owl/PO#PO_0006016";
        final String taggedResourceType = "http://www.podd.org/poddModel#Measurement";
        
        final String rdfJsonDocument =
                TestUtils.createTestAnnotationDocument(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), null,
                        taggedResourceUri, taggedResourceType, ontologyUri, tagName, authorUri);
        
        final Representation input =
                new AppendableRepresentation(rdfJsonDocument, RestletUtilMediaType.APPLICATION_RDF_JSON,
                        Language.DEFAULT, CharacterSet.UTF_8);
        
        final Representation createRepresentation =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_JSON, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        
        final Map<URI, Annotation> parsedResultAnnotations =
                SesameUtils.parseAnnotations(createRepresentation.getReader(), this.getUrl("/"),
                        RestletUtilMediaType.APPLICATION_RDF_JSON.getName());
        
        Assert.assertEquals(1, parsedResultAnnotations.size());
        Assert.assertEquals(1, parsedResultAnnotations.keySet().size());
        Assert.assertEquals(1, parsedResultAnnotations.values().size());
        
        for(final URI nextUri : parsedResultAnnotations.keySet())
        {
            final Annotation parsedResultAnnotation = parsedResultAnnotations.get(nextUri);
            
            // Now try to fetch that annotation again to test that it was stored correctly
            final ClientResource checkingClientResource =
                    new ClientResource(this.getAnnotationUrl(parsedResultAnnotation));
            
            final Representation checkingRepresentation =
                    checkingClientResource.get(RestletUtilMediaType.APPLICATION_RDF_JSON);
            
            Assert.assertEquals(200, checkingClientResource.getStatus().getCode());
            
            Assert.assertEquals(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), checkingRepresentation
                    .getMediaType().toString());
            
            final Map<URI, Annotation> refetchedResultAnnotations =
                    SesameUtils.parseAnnotations(checkingRepresentation.getReader(), this.getUrl("/"),
                            RestletUtilMediaType.APPLICATION_RDF_JSON.getName());
            
            Assert.assertEquals(1, refetchedResultAnnotations.size());
            Assert.assertEquals(1, refetchedResultAnnotations.keySet().size());
            Assert.assertEquals(1, refetchedResultAnnotations.values().size());
            
            for(final URI nextRefetchedUri : refetchedResultAnnotations.keySet())
            {
                final Annotation refetchedResultAnnotation = refetchedResultAnnotations.get(nextRefetchedUri);
                
                // Check each of the properties to verify that they matched the original
                // properties
                Assert.assertEquals(refetchedResultAnnotation.getAnnotationUniqueId(),
                        parsedResultAnnotation.getAnnotationUniqueId());
                Assert.assertEquals(refetchedResultAnnotation.getAnnotatedObjectType(),
                        parsedResultAnnotation.getAnnotatedObjectType());
                Assert.assertEquals(refetchedResultAnnotation.getAnnotatedObjectUri(),
                        parsedResultAnnotation.getAnnotatedObjectUri());
                Assert.assertEquals(refetchedResultAnnotation.getAnnotator(), parsedResultAnnotation.getAnnotator());
                Assert.assertEquals(refetchedResultAnnotation.getOntologyTermUri(),
                        parsedResultAnnotation.getOntologyTermUri());
                Assert.assertEquals(refetchedResultAnnotation.getTag(), parsedResultAnnotation.getTag());
            }
            
            // Get the resource and dump it to the console
            final ClientResource deleteClientResource =
                    new ClientResource(this.getAnnotationUrl(parsedResultAnnotation));
            
            final Representation representation =
                    RestletTestUtils.doTestAuthenticatedRequest(deleteClientResource, Method.DELETE, input,
                            MediaType.APPLICATION_JSON, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
        }
    }
    
    /**
     * Test whether Annotations can be stored and retrieved using the application with an RDF/XML
     * request and an HTML response.
     */
    @Test
    public void testAnnotationCreationJson2JsonNoAuthor() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        final String taggedResourceUri = "http://example.org/podd/sample/1";
        // final String authorUri = "http://example.org/peter";
        final String authorUri = null;
        final String tagName = "leaf measurement";
        final String ontologyUri = "http://purl.org/obo/owl/PO#PO_0006016";
        final String taggedResourceType = "http://www.podd.org/poddModel#Measurement";
        
        final String rdfJsonDocument =
                TestUtils.createTestAnnotationDocument(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), null,
                        taggedResourceUri, taggedResourceType, ontologyUri, tagName, authorUri);
        
        final Representation input =
                new AppendableRepresentation(rdfJsonDocument, RestletUtilMediaType.APPLICATION_RDF_JSON,
                        Language.DEFAULT, CharacterSet.UTF_8);
        
        final Representation createRepresentation =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_JSON, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        
        final Map<URI, Annotation> parsedResultAnnotations =
                SesameUtils.parseAnnotations(createRepresentation.getReader(), this.getUrl("/"),
                        RestletUtilMediaType.APPLICATION_RDF_JSON.getName());
        
        Assert.assertEquals(1, parsedResultAnnotations.size());
        Assert.assertEquals(1, parsedResultAnnotations.keySet().size());
        Assert.assertEquals(1, parsedResultAnnotations.values().size());
        
        for(final URI nextUri : parsedResultAnnotations.keySet())
        {
            final Annotation parsedResultAnnotation = parsedResultAnnotations.get(nextUri);
            
            // Now try to fetch that annotation again to test that it was stored correctly
            final ClientResource checkingClientResource =
                    new ClientResource(this.getAnnotationUrl(parsedResultAnnotation));
            
            final Representation checkingRepresentation =
                    checkingClientResource.get(RestletUtilMediaType.APPLICATION_RDF_JSON);
            
            Assert.assertEquals(200, checkingClientResource.getStatus().getCode());
            
            Assert.assertEquals(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), checkingRepresentation
                    .getMediaType().toString());
            
            final Map<URI, Annotation> refetchedResultAnnotations =
                    SesameUtils.parseAnnotations(checkingRepresentation.getReader(), this.getUrl("/"),
                            RestletUtilMediaType.APPLICATION_RDF_JSON.getName());
            
            Assert.assertEquals(1, refetchedResultAnnotations.size());
            Assert.assertEquals(1, refetchedResultAnnotations.keySet().size());
            Assert.assertEquals(1, refetchedResultAnnotations.values().size());
            
            for(final URI nextRefetchedUri : refetchedResultAnnotations.keySet())
            {
                final Annotation refetchedResultAnnotation = refetchedResultAnnotations.get(nextRefetchedUri);
                
                // Check each of the properties to verify that they matched the original
                // properties
                Assert.assertEquals(refetchedResultAnnotation.getAnnotationUniqueId(),
                        parsedResultAnnotation.getAnnotationUniqueId());
                Assert.assertEquals(refetchedResultAnnotation.getAnnotatedObjectType(),
                        parsedResultAnnotation.getAnnotatedObjectType());
                Assert.assertEquals(refetchedResultAnnotation.getAnnotatedObjectUri(),
                        parsedResultAnnotation.getAnnotatedObjectUri());
                Assert.assertEquals(refetchedResultAnnotation.getAnnotator(), parsedResultAnnotation.getAnnotator());
                Assert.assertEquals(refetchedResultAnnotation.getOntologyTermUri(),
                        parsedResultAnnotation.getOntologyTermUri());
                Assert.assertEquals(refetchedResultAnnotation.getTag(), parsedResultAnnotation.getTag());
            }
            
            // Get the resource and dump it to the console
            final ClientResource deleteClientResource =
                    new ClientResource(this.getAnnotationUrl(parsedResultAnnotation));
            
            final Representation representation =
                    RestletTestUtils.doTestAuthenticatedRequest(deleteClientResource, Method.DELETE, input,
                            MediaType.APPLICATION_JSON, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
        }
    }
    
    /**
     * Test whether Annotations can be stored and retrieved using the application with an RDF/XML
     * request and an HTML response.
     */
    @Test
    public void testAnnotationCreationRdfXml2Html() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        final String taggedResourceUri = "http://example.org/podd/sample/1";
        final String authorUri = "http://example.org/peter";
        final String tagName = "leaf measurement";
        final String ontologyUri = "http://purl.org/obo/owl/PO#PO_0006016";
        final String taggedResourceType = "http://www.podd.org/poddModel#Measurement";
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestAnnotationDocument("application/rdf+xml", null, taggedResourceUri,
                        taggedResourceType, ontologyUri, tagName, authorUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.TEXT_HTML, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        
        final String resultsText = results.getText();
        this.assertFreemarker(resultsText);
        Assert.assertTrue(resultsText.contains("OAS-webservice-impl Annotations list template"));
        Assert.assertFalse(resultsText.contains("There are no matching annotations for your query"));
        Assert.assertTrue(resultsText.contains("List of Annotations"));
        Assert.assertTrue(resultsText.contains("class=\"annotation\""));
        Assert.assertFalse(resultsText.contains("The annotation could not be found"));
    }
    
    /**
     * Test whether Annotations can be stored and retrieved using the application with an RDF/XML
     * request and an HTML response.
     */
    @Test
    public void testAnnotationCreationRdfXml2Json() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        final String taggedResourceUri = "http://example.org/podd/sample/1";
        final String authorUri = "http://example.org/peter";
        final String tagName = "leaf measurement";
        final String ontologyUri = "http://purl.org/obo/owl/PO#PO_0006016";
        final String taggedResourceType = "http://www.podd.org/poddModel#Measurement";
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestAnnotationDocument(MediaType.APPLICATION_RDF_XML.getName(), null,
                        taggedResourceUri, taggedResourceType, ontologyUri, tagName, authorUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_JSON, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
    }
    
    /**
     * Test whether Annotations can be stored and retrieved using the application with both an
     * RDF/XML request and response.
     */
    @Test
    public void testAnnotationCreationRdfXml2RdfXml() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        final String taggedResourceUri = "http://example.org/podd/sample/1";
        final String authorUri = "http://example.org/peter";
        final String tagName = "leaf measurement";
        final String ontologyUri = "http://purl.org/obo/owl/PO#PO_0006016";
        final String taggedResourceType = "http://www.podd.org/poddModel#Measurement";
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestAnnotationDocument("application/rdf+xml", null, taggedResourceUri,
                        taggedResourceType, ontologyUri, tagName, authorUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        final Representation createRepresentation =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        
    }
    
    /**
     * Test whether Annotations can be stored and retrieved using the application with an RDF/XML
     * request and an HTML response.
     */
    @Test
    public void testAnnotationUpdatesJson2Json() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        final String taggedResourceUri = "http://example.org/podd/sample/1";
        String authorUri = "http://example.org/peter";
        final String tagName = "leaf measurement";
        final String ontologyUri = "http://purl.org/obo/owl/PO#PO_0006016";
        final String taggedResourceType = "http://www.podd.org/poddModel#Measurement";
        
        final String rdfJsonDocument =
                TestUtils.createTestAnnotationDocument(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), null,
                        taggedResourceUri, taggedResourceType, ontologyUri, tagName, authorUri);
        
        final Representation input =
                new AppendableRepresentation(rdfJsonDocument, RestletUtilMediaType.APPLICATION_RDF_JSON,
                        Language.DEFAULT, CharacterSet.UTF_8);
        
        final Representation createRepresentation =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_JSON, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        
        final Map<URI, Annotation> parsedResultAnnotations =
                SesameUtils.parseAnnotations(createRepresentation.getReader(), this.getUrl("/"),
                        RestletUtilMediaType.APPLICATION_RDF_JSON.getName());
        
        Assert.assertEquals(1, parsedResultAnnotations.size());
        Assert.assertEquals(1, parsedResultAnnotations.keySet().size());
        Assert.assertEquals(1, parsedResultAnnotations.values().size());
        
        for(final URI nextUri : parsedResultAnnotations.keySet())
        {
            final Annotation parsedResultAnnotation = parsedResultAnnotations.get(nextUri);
            
            final String annotationUniqueId = parsedResultAnnotation.getAnnotationUniqueId().stringValue();
            
            final ClientResource updateClientResource =
                    new ClientResource(this.getAnnotationUrl(parsedResultAnnotation));
            
            // Change the author URI to test the update facility
            authorUri = "http://example.org/jimbob";
            
            // create a new RDF/XML document using the annotationUniqueId that was returned by
            // the
            // first HTTP PUT
            final String rdfXmlUpdateDocument =
                    TestUtils.createTestAnnotationDocument("application/rdf+xml", annotationUniqueId,
                            taggedResourceUri, taggedResourceType, ontologyUri, tagName, authorUri);
            
            final Representation updateInput =
                    new AppendableRepresentation(rdfXmlUpdateDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            final Representation updateRepresentation =
                    RestletTestUtils.doTestAuthenticatedRequest(updateClientResource, Method.PUT, updateInput,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
            
            final Map<URI, Annotation> parsedUpdateAnnotations =
                    SesameUtils.parseAnnotations(updateRepresentation.getReader(), this.getUrl("/"),
                            MediaType.APPLICATION_RDF_XML.getName());
            
            Assert.assertEquals(1, parsedUpdateAnnotations.size());
            Assert.assertEquals(1, parsedUpdateAnnotations.keySet().size());
            Assert.assertEquals(1, parsedUpdateAnnotations.values().size());
            
            for(final URI nextUpdateUri : parsedUpdateAnnotations.keySet())
            {
                final Annotation parsedUpdateAnnotation = parsedUpdateAnnotations.get(nextUpdateUri);
                // Check each of the unchanged properties to verify that they matched the
                // original
                // properties
                Assert.assertEquals(parsedUpdateAnnotation.getAnnotationUniqueId(),
                        parsedResultAnnotation.getAnnotationUniqueId());
                Assert.assertEquals(parsedUpdateAnnotation.getAnnotatedObjectType(),
                        parsedResultAnnotation.getAnnotatedObjectType());
                Assert.assertEquals(parsedUpdateAnnotation.getAnnotatedObjectUri(),
                        parsedResultAnnotation.getAnnotatedObjectUri());
                Assert.assertEquals(parsedUpdateAnnotation.getOntologyTermUri(),
                        parsedResultAnnotation.getOntologyTermUri());
                Assert.assertEquals(parsedUpdateAnnotation.getTag(), parsedResultAnnotation.getTag());
                
                // check that the annotator was not the same as the original
                Assert.assertNotSame(parsedUpdateAnnotation.getAnnotator(), parsedResultAnnotation.getAnnotator());
                // check that the annotator was correctly updated
                Assert.assertEquals(parsedUpdateAnnotation.getAnnotator().stringValue(), authorUri);
            }
            
            // Get the resource and dump it to the console
            final ClientResource deleteClientResource =
                    new ClientResource(this.getAnnotationUrl(parsedResultAnnotation));
            
            final Representation representation =
                    RestletTestUtils.doTestAuthenticatedRequest(deleteClientResource, Method.DELETE, input,
                            MediaType.APPLICATION_JSON, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
        }
    }
    
    /**
     * Test whether Annotations can be stored and retrieved using the application.
     */
    @Test
    public void testUnknownAnnotationDeletion()
    {
        try
        {
            // Get the resource and dump it to the console
            final ClientResource cr =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_PATH,
                            OasProps.DEF_ANNOTATION_PATH) + "?annotationId=dummyid"));
            
            @SuppressWarnings("unused")
            final Representation representation = cr.delete();
            
            Assert.assertEquals(400, cr.getResponse().getStatus().getCode());
            
            Assert.fail("Did not find expected exception when deleting resource that did not exist");
        }
        catch(final Exception ex)
        {
            
        }
        
    }
    
}
