/**
 * OAS Webservice test implementations
 */
package com.github.ansell.oas.webservice.impl.test;

import java.util.Map;

import org.junit.Assert;
import org.junit.Test;
import org.openrdf.model.URI;
import org.restlet.data.CharacterSet;
import org.restlet.data.Language;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.representation.AppendableRepresentation;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;

import com.github.ansell.oas.objects.Annotation;
import com.github.ansell.oas.objects.ResourceCount;
import com.github.ansell.oas.test.TestUtils;
import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.oas.utils.RestletUtils;
import com.github.ansell.oas.utils.SesameUtils;
import com.github.ansell.restletutils.RestletUtilMediaType;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * Unit test for AnnotationResourceImpl.
 * 
 * Setup and Teardown methods are implemented in AbstractResourceImplTest.
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public class AnnotationCountsByOntologyTermUriResourceImplTest extends AbstractResourceImplTest
{
    /**
     * Test whether Annotations can be stored and retrieved using the application with a JSON
     * request and a JSON response.
     */
    @Test
    public void testAnnotationCreationAndFetchingByOntologyTermUriJson() throws Exception
    {
        // create a new annotation
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        final String taggedResourceUri = "http://example.org/podd/sample/1";
        final String authorUri = "http://example.org/peter";
        final String tagName = "leaf measurement";
        final String ontologyTermUri = "http://purl.org/obo/owl/PO#PO_0006016";
        final String taggedResourceType = "http://www.podd.org/poddModel#Measurement";
        
        final String rdfJsonDocument =
                TestUtils.createTestAnnotationDocument(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), null,
                        taggedResourceUri, taggedResourceType, ontologyTermUri, tagName, authorUri);
        
        final Representation input =
                new AppendableRepresentation(rdfJsonDocument, RestletUtilMediaType.APPLICATION_RDF_JSON,
                        Language.DEFAULT, CharacterSet.UTF_8);
        
        final Representation createRepresentation =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_JSON, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        
        final Map<URI, Annotation> parsedResultAnnotations =
                SesameUtils.parseAnnotations(createRepresentation.getReader(), this.getUrl("/"), createRepresentation
                        .getMediaType().getName());
        
        Assert.assertEquals(1, parsedResultAnnotations.size());
        Assert.assertEquals(1, parsedResultAnnotations.keySet().size());
        Assert.assertEquals(1, parsedResultAnnotations.values().size());
        
        for(final URI nextAnnotationUri : parsedResultAnnotations.keySet())
        {
            final Annotation parsedResultAnnotation = parsedResultAnnotations.get(nextAnnotationUri);
            Assert.assertEquals(taggedResourceType, parsedResultAnnotation.getAnnotatedObjectType().stringValue());
            Assert.assertEquals(taggedResourceUri, parsedResultAnnotation.getAnnotatedObjectUri().stringValue());
            Assert.assertEquals(authorUri, parsedResultAnnotation.getAnnotator().stringValue());
            Assert.assertEquals(ontologyTermUri, parsedResultAnnotation.getOntologyTermUri().stringValue());
            Assert.assertEquals(tagName, parsedResultAnnotation.getTag().stringValue());
        }
        // Now test the Annotations By Object Type Web Service
        final ClientResource getAnnotationCountsByOntologyTermUriClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(
                        OasProps.PROP_ANNOTATION_COUNTS_BY_ONTOLOGY_TERM_URI_PATH,
                        OasProps.DEF_ANNOTATION_COUNTS_BY_ONTOLOGY_TERM_URI_PATH)));
        
        getAnnotationCountsByOntologyTermUriClientResource.getReference().addQueryParameter("ontologytermuri",
                ontologyTermUri);
        
        final Representation countRepresentation =
                getAnnotationCountsByOntologyTermUriClientResource.get(RestletUtilMediaType.APPLICATION_RDF_JSON);
        
        Assert.assertEquals(200, getAnnotationCountsByOntologyTermUriClientResource.getStatus().getCode());
        
        Assert.assertEquals(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), countRepresentation.getMediaType()
                .getName());
        
        final ResourceCount annotationCount =
                SesameUtils.parseUriCounts(countRepresentation.getReader(), this.getUrl("/"), countRepresentation
                        .getMediaType().getName());
        
        // Check each of the properties to verify that they matched the original properties
        Assert.assertEquals(1, annotationCount.getCount());
        
        for(final URI nextAnnotationUri : parsedResultAnnotations.keySet())
        {
            final Annotation parsedResultAnnotation = parsedResultAnnotations.get(nextAnnotationUri);
            
            // cleanup by deleting the resource
            final ClientResource deleteClientResource =
                    new ClientResource(this.getAnnotationUrl(parsedResultAnnotation));
            
            final Representation representation =
                    RestletTestUtils.doTestAuthenticatedRequest(deleteClientResource, Method.DELETE, input,
                            MediaType.APPLICATION_JSON, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
        }
    }
    
    /**
     * Test whether Annotations can be stored and retrieved using the application with a JSON
     * request and a JSON response.
     */
    @Test
    public void testAnnotationCreationAndFetchingByOntologyTermUriRdfXml() throws Exception
    {
        // create a new annotation
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        final String taggedResourceUri = "http://example.org/podd/sample/1";
        String authorUri = "http://example.org/peter";
        final String tagName = "leaf measurement";
        final String ontologyTermUri = "http://purl.org/obo/owl/PO#PO_0006016";
        final String taggedResourceType = "http://www.podd.org/poddModel#Measurement";
        
        final String rdfDocument =
                TestUtils.createTestAnnotationDocument(MediaType.APPLICATION_RDF_XML.getName(), null,
                        taggedResourceUri, taggedResourceType, ontologyTermUri, tagName, authorUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        final Representation createRepresentation =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        
        final Map<URI, Annotation> parsedResultAnnotations =
                SesameUtils.parseAnnotations(createRepresentation.getReader(), this.getUrl("/"),
                        MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, parsedResultAnnotations.size());
        Assert.assertEquals(1, parsedResultAnnotations.keySet().size());
        Assert.assertEquals(1, parsedResultAnnotations.values().size());
        
        for(final URI nextAnnotationUri : parsedResultAnnotations.keySet())
        {
            final Annotation parsedResultAnnotation = parsedResultAnnotations.get(nextAnnotationUri);
            Assert.assertEquals(taggedResourceType, parsedResultAnnotation.getAnnotatedObjectType().stringValue());
            Assert.assertEquals(taggedResourceUri, parsedResultAnnotation.getAnnotatedObjectUri().stringValue());
            Assert.assertEquals(authorUri, parsedResultAnnotation.getAnnotator().stringValue());
            Assert.assertEquals(ontologyTermUri, parsedResultAnnotation.getOntologyTermUri().stringValue());
            Assert.assertEquals(tagName, parsedResultAnnotation.getTag().stringValue());
        }
        
        // create a new annotation
        final ClientResource secondCreationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        authorUri = "http://example.org/gavin";
        
        final String secondRdfDocument =
                TestUtils.createTestAnnotationDocument(MediaType.APPLICATION_RDF_XML.getName(), null,
                        taggedResourceUri, taggedResourceType, ontologyTermUri, tagName, authorUri);
        
        final Representation secondInput =
                new AppendableRepresentation(secondRdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        final Representation secondCreateRepresentation =
                RestletTestUtils.doTestAuthenticatedRequest(secondCreationClientResource, Method.POST, secondInput,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        
        final Map<URI, Annotation> secondParsedResultAnnotations =
                SesameUtils.parseAnnotations(secondCreateRepresentation.getReader(), this.getUrl("/"),
                        MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, secondParsedResultAnnotations.size());
        Assert.assertEquals(1, secondParsedResultAnnotations.keySet().size());
        Assert.assertEquals(1, secondParsedResultAnnotations.values().size());
        
        for(final URI nextAnnotationUri : secondParsedResultAnnotations.keySet())
        {
            final Annotation secondParsedResultAnnotation = secondParsedResultAnnotations.get(nextAnnotationUri);
            
            Assert.assertEquals(taggedResourceType, secondParsedResultAnnotation.getAnnotatedObjectType().stringValue());
            Assert.assertEquals(taggedResourceUri, secondParsedResultAnnotation.getAnnotatedObjectUri().stringValue());
            Assert.assertEquals(authorUri, secondParsedResultAnnotation.getAnnotator().stringValue());
            Assert.assertEquals(ontologyTermUri, secondParsedResultAnnotation.getOntologyTermUri().stringValue());
            Assert.assertEquals(tagName, secondParsedResultAnnotation.getTag().stringValue());
        }
        // Now test the Annotations By Object Type Web Service
        final ClientResource getAnnotationCountsByOntologyTermUriClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(
                        OasProps.PROP_ANNOTATION_COUNTS_BY_ONTOLOGY_TERM_URI_PATH,
                        OasProps.DEF_ANNOTATION_COUNTS_BY_ONTOLOGY_TERM_URI_PATH)));
        
        getAnnotationCountsByOntologyTermUriClientResource.getReference().addQueryParameter("ontologytermuri",
                ontologyTermUri);
        
        final Representation countRepresentation =
                getAnnotationCountsByOntologyTermUriClientResource.get(MediaType.APPLICATION_RDF_XML);
        
        Assert.assertEquals(200, getAnnotationCountsByOntologyTermUriClientResource.getStatus().getCode());
        
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), countRepresentation.getMediaType().getName());
        
        final ResourceCount annotationCount =
                SesameUtils.parseUriCounts(countRepresentation.getReader(), this.getUrl("/"), countRepresentation
                        .getMediaType().getName());
        
        System.out.print(RestletUtils.toRDFSerialisation("application/rdf+xml",
                SesameUtils.toRDFRepository(annotationCount, null)).getText());
        
        // Check each of the properties to verify that they matched the original properties
        Assert.assertEquals(2, annotationCount.getCount());
        
        for(final URI nextAnnotationUri : parsedResultAnnotations.keySet())
        {
            final Annotation parsedResultAnnotation = parsedResultAnnotations.get(nextAnnotationUri);
            // cleanup by deleting the resource
            final ClientResource deleteClientResource =
                    new ClientResource(this.getAnnotationUrl(parsedResultAnnotation));
            
            final Representation representation =
                    RestletTestUtils.doTestAuthenticatedRequest(deleteClientResource, Method.DELETE, input,
                            MediaType.APPLICATION_JSON, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
        }
        
        for(final URI nextAnnotationUri : secondParsedResultAnnotations.keySet())
        {
            final Annotation secondParsedResultAnnotation = secondParsedResultAnnotations.get(nextAnnotationUri);
            // cleanup by deleting the resource
            final ClientResource deleteClientResource =
                    new ClientResource(this.getAnnotationUrl(secondParsedResultAnnotation));
            
            final Representation representation =
                    RestletTestUtils.doTestAuthenticatedRequest(deleteClientResource, Method.DELETE, input,
                            MediaType.APPLICATION_JSON, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
        }
    }
}
