/**
 * OAS Webservice test implementations
 */
package com.github.ansell.oas.webservice.impl.test;

import java.util.Map;

import org.junit.Assert;
import org.junit.Test;
import org.openrdf.model.URI;
import org.restlet.data.CharacterSet;
import org.restlet.data.Language;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.representation.AppendableRepresentation;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;

import com.github.ansell.oas.objects.Annotation;
import com.github.ansell.oas.test.TestUtils;
import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.oas.utils.SesameUtils;
import com.github.ansell.restletutils.RestletUtilMediaType;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * Unit test for AnnotationResourceImpl
 * 
 * Setup and Teardown methods are implemented in AbstractResourceImplTest
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public class AnnotationsByAnnotatedObjectTypeResourceImplTest extends AbstractResourceImplTest
{
    
    /**
     * Test whether Annotations can be stored and retrieved using the application with an RDF/XML
     * request and an HTML response
     */
    @Test
    public void testAnnotationCreationAndFetchingByAnnotatedObjectType() throws Exception
    {
        // create a new annotation
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        final String taggedResourceUri = "http://example.org/podd/sample/1";
        final String authorUri = "http://example.org/peter";
        final String tagName = "leaf measurement";
        final String ontologyUri = "http://purl.org/obo/owl/PO#PO_0006016";
        final String taggedResourceType = "http://www.podd.org/poddModel#Measurement";
        
        final String rdfJsonDocument =
                TestUtils.createTestAnnotationDocument(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), null,
                        taggedResourceUri, taggedResourceType, ontologyUri, tagName, authorUri);
        
        final Representation input =
                new AppendableRepresentation(rdfJsonDocument, RestletUtilMediaType.APPLICATION_RDF_JSON,
                        Language.DEFAULT, CharacterSet.UTF_8);
        
        final Representation createRepresentation =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_JSON, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        
        final Map<URI, Annotation> parsedResultAnnotations =
                SesameUtils.parseAnnotations(createRepresentation.getReader(), this.getUrl("/"),
                        RestletUtilMediaType.APPLICATION_RDF_JSON.getName());
        
        Assert.assertEquals(1, parsedResultAnnotations.size());
        Assert.assertEquals(1, parsedResultAnnotations.keySet().size());
        Assert.assertEquals(1, parsedResultAnnotations.values().size());
        
        for(final URI nextUri : parsedResultAnnotations.keySet())
        {
            final Annotation parsedResultAnnotation = parsedResultAnnotations.get(nextUri);
            
            Assert.assertEquals(taggedResourceType, parsedResultAnnotation.getAnnotatedObjectType().stringValue());
            Assert.assertEquals(taggedResourceUri, parsedResultAnnotation.getAnnotatedObjectUri().stringValue());
            Assert.assertEquals(authorUri, parsedResultAnnotation.getAnnotator().stringValue());
            Assert.assertEquals(ontologyUri, parsedResultAnnotation.getOntologyTermUri().stringValue());
            Assert.assertEquals(tagName, parsedResultAnnotation.getTag().stringValue());
            
            // Now test the Annotations By Object Type Web Service
            final ClientResource getAnnotationsByObjectTypeClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATIONS_BY_OBJECT_TYPE_PATH,
                            OasProps.DEF_ANNOTATIONS_BY_OBJECT_TYPE_PATH)));
            
            getAnnotationsByObjectTypeClientResource.getReference().addQueryParameter("objecttype", taggedResourceType);
            
            final Representation checkingRepresentation =
                    getAnnotationsByObjectTypeClientResource.get(RestletUtilMediaType.APPLICATION_RDF_JSON);
            
            Assert.assertEquals(200, getAnnotationsByObjectTypeClientResource.getStatus().getCode());
            
            Assert.assertEquals(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), checkingRepresentation
                    .getMediaType().toString());
            
            final Map<URI, Annotation> refetchedResultAnnotations =
                    SesameUtils.parseAnnotations(checkingRepresentation.getReader(), this.getUrl("/"),
                            RestletUtilMediaType.APPLICATION_RDF_JSON.getName());
            
            Assert.assertEquals(1, refetchedResultAnnotations.size());
            Assert.assertEquals(1, refetchedResultAnnotations.keySet().size());
            Assert.assertEquals(1, refetchedResultAnnotations.values().size());
            
            for(final URI nextRefetchedUri : refetchedResultAnnotations.keySet())
            {
                final Annotation refetchedResultAnnotation = refetchedResultAnnotations.get(nextRefetchedUri);
                
                // Check each of the properties to verify that they matched the original
                // properties
                Assert.assertEquals(refetchedResultAnnotation.getAnnotationUniqueId(),
                        parsedResultAnnotation.getAnnotationUniqueId());
                Assert.assertEquals(refetchedResultAnnotation.getAnnotatedObjectType(),
                        parsedResultAnnotation.getAnnotatedObjectType());
                Assert.assertEquals(refetchedResultAnnotation.getAnnotatedObjectUri(),
                        parsedResultAnnotation.getAnnotatedObjectUri());
                
                // These properties should not be in the condensed RDF tripleset that is
                // returned by
                // this search
                Assert.assertNull(refetchedResultAnnotation.getAnnotator());
                Assert.assertNull(refetchedResultAnnotation.getOntologyTermUri());
                Assert.assertNull(refetchedResultAnnotation.getTag());
            }
            
            // cleanup by deleting the resource
            final ClientResource deleteClientResource =
                    new ClientResource(this.getAnnotationUrl(parsedResultAnnotation));
            
            final Representation representation =
                    RestletTestUtils.doTestAuthenticatedRequest(deleteClientResource, Method.DELETE, input,
                            MediaType.APPLICATION_JSON, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
        }
    }
    
}
