/**
 * OAS Webservice test implementations
 */
package com.github.ansell.oas.webservice.impl.test;

import org.junit.Assert;
import org.junit.Test;
import org.openrdf.model.Model;
import org.openrdf.model.vocabulary.RDF;
import org.openrdf.rio.RDFFormat;
import org.restlet.data.MediaType;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;

import com.github.ansell.oas.utils.OasConstants;
import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.restletutils.RestletUtilMediaType;

/**
 * Unit test for BulkFetchAnnotationResourceImpl.
 */
public class BulkFetchAnnotationResourceImplTest extends AbstractResourceImplTest
{
    /**
     * Test whether the HTML page when there are no annotations is correct.
     */
    @Test
    public void testAnnotationListEmptyHtml() throws Exception
    {
        final ClientResource getResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_BULK_FETCH_ANNOTATION_PATH,
                        OasProps.DEF_BULK_FETCH_ANNOTATION_PATH)));
        
        final Representation results = getResource.get(MediaType.TEXT_HTML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getResource.getResponse().getStatus().getCode());
        // Expecting application/json MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), results.getMediaType().getName());
        
        final String resultsText = results.getText();
        this.assertFreemarker(resultsText);
        
        Assert.assertTrue(resultsText.contains("OAS-webservice-impl Annotations list template"));
        Assert.assertTrue(resultsText.contains("There are no matching annotations for your query"));
        Assert.assertFalse(resultsText.contains("List of Annotations"));
        Assert.assertFalse(resultsText.contains("class=\"annotation\""));
    }
    
    /**
     * Tests whether the RDF/JSON page when there are no annotations is correct.
     */
    @Test
    public void testAnnotationListEmptyJson() throws Exception
    {
        final ClientResource getResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_BULK_FETCH_ANNOTATION_PATH,
                        OasProps.DEF_BULK_FETCH_ANNOTATION_PATH)));
        
        final Representation results = getResource.get(RestletUtilMediaType.APPLICATION_RDF_JSON);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getResource.getResponse().getStatus().getCode());
        // Expecting application/json MIME type for response
        Assert.assertEquals(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), results.getMediaType().getName());
        
        this.assertRdf(results.getStream(), RDFFormat.RDFJSON, 0);
    }
    
    /**
     * Tests whether the RDF/XML page when there are no annotations is correct.
     */
    @Test
    public void testAnnotationListEmptyRdfXml() throws Exception
    {
        final ClientResource getResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_BULK_FETCH_ANNOTATION_PATH,
                        OasProps.DEF_BULK_FETCH_ANNOTATION_PATH)));
        
        final Representation results = getResource.get(MediaType.APPLICATION_RDF_XML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getResource.getResponse().getStatus().getCode());
        // Expecting application/rdf+xml MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        this.assertRdf(results.getStream(), RDFFormat.RDFXML, 0);
    }
    
    /**
     * Test whether Annotations can be listed using an HTTP GET request to retrieve HTML.
     */
    @Test
    public void testAnnotationListMultipleHtml() throws Exception
    {
        this.createTestAnnotations(50);
        
        final ClientResource getResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_BULK_FETCH_ANNOTATION_PATH,
                        OasProps.DEF_BULK_FETCH_ANNOTATION_PATH)));
        
        final Representation results = getResource.get(MediaType.TEXT_HTML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getResource.getResponse().getStatus().getCode());
        // Expecting application/json MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), results.getMediaType().getName());
        
        final String resultsText = results.getText();
        this.assertFreemarker(resultsText);
        Assert.assertTrue(resultsText.contains("OAS-webservice-impl Annotations list template"));
        Assert.assertFalse(resultsText.contains("There are no matching annotations for your query"));
        Assert.assertTrue(resultsText.contains("List of Annotations"));
        Assert.assertTrue(resultsText.contains("class=\"annotation\""));
    }
    
    /**
     * Test whether Annotations can be listed using an HTTP GET request to retrieve RDF/JSON.
     */
    @Test
    public void testAnnotationListMultipleJson() throws Exception
    {
        this.createTestAnnotations(50);
        
        final ClientResource getResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_BULK_FETCH_ANNOTATION_PATH,
                        OasProps.DEF_BULK_FETCH_ANNOTATION_PATH)));
        
        final Representation results = getResource.get(RestletUtilMediaType.APPLICATION_RDF_JSON);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getResource.getResponse().getStatus().getCode());
        // Expecting application/json MIME type for response
        Assert.assertEquals(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), results.getMediaType().getName());
        
        final Model resultModel = this.assertRdf(results.getStream(), RDFFormat.RDFJSON, 300);
        
        Assert.assertTrue(resultModel.contains(null, RDF.TYPE, OasConstants.TAGGING_TAGGING));
    }
    
    /**
     * Test whether Annotations can be listed using an HTTP GET request to retrieve RDF/XML.
     */
    @Test
    public void testAnnotationListMultipleRdfXml() throws Exception
    {
        this.createTestAnnotations(50);
        
        final ClientResource getResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_BULK_FETCH_ANNOTATION_PATH,
                        OasProps.DEF_BULK_FETCH_ANNOTATION_PATH)));
        
        final Representation results = getResource.get(MediaType.APPLICATION_RDF_XML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getResource.getResponse().getStatus().getCode());
        // Expecting application/json MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final Model resultModel = this.assertRdf(results.getStream(), RDFFormat.RDFXML, 300);
        
        Assert.assertTrue(resultModel.contains(null, RDF.TYPE, OasConstants.TAGGING_TAGGING));
    }
    
    /**
     * Test whether Annotations can be listed using an HTTP GET request to retrieve HTML.
     */
    @Test
    public void testAnnotationListSingleHtml() throws Exception
    {
        this.createTestAnnotations(1);
        
        final ClientResource getResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_BULK_FETCH_ANNOTATION_PATH,
                        OasProps.DEF_BULK_FETCH_ANNOTATION_PATH)));
        
        final Representation results = getResource.get(MediaType.TEXT_HTML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getResource.getResponse().getStatus().getCode());
        // Expecting application/json MIME type for response
        Assert.assertEquals(MediaType.TEXT_HTML.getName(), results.getMediaType().getName());
        
        final String resultsText = results.getText();
        this.assertFreemarker(resultsText);
        Assert.assertTrue(resultsText.contains("OAS-webservice-impl Annotations list template"));
        Assert.assertFalse(resultsText.contains("There are no matching annotations for your query"));
        Assert.assertTrue(resultsText.contains("List of Annotations"));
        Assert.assertTrue(resultsText.contains("class=\"annotation\""));
    }
    
    /**
     * Test whether Annotations can be listed using an HTTP GET request to retrieve RDF/JSON.
     */
    @Test
    public void testAnnotationListSingleJson() throws Exception
    {
        this.createTestAnnotations(1);
        
        final ClientResource getResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_BULK_FETCH_ANNOTATION_PATH,
                        OasProps.DEF_BULK_FETCH_ANNOTATION_PATH)));
        
        final Representation results = getResource.get(RestletUtilMediaType.APPLICATION_RDF_JSON);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getResource.getResponse().getStatus().getCode());
        // Expecting application/json MIME type for response
        Assert.assertEquals(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), results.getMediaType().getName());
        
        final Model resultModel = this.assertRdf(results.getStream(), RDFFormat.RDFJSON, 6);
        
        Assert.assertTrue(resultModel.contains(null, RDF.TYPE, OasConstants.TAGGING_TAGGING));
    }
    
    /**
     * Test whether Annotations can be listed using an HTTP GET request to retrieve RDF/XML.
     */
    @Test
    public void testAnnotationListSingleRdfXml() throws Exception
    {
        this.createTestAnnotations(1);
        
        final ClientResource getResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_BULK_FETCH_ANNOTATION_PATH,
                        OasProps.DEF_BULK_FETCH_ANNOTATION_PATH)));
        
        final Representation results = getResource.get(MediaType.APPLICATION_RDF_XML);
        
        // Expecting HTTP 200 OK response
        Assert.assertEquals(200, getResource.getResponse().getStatus().getCode());
        // Expecting application/json MIME type for response
        Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
        
        final Model resultModel = this.assertRdf(results.getStream(), RDFFormat.RDFXML, 6);
        
        Assert.assertTrue(resultModel.contains(null, RDF.TYPE, OasConstants.TAGGING_TAGGING));
    }
    
}
