/**
 * 
 */
package com.github.ansell.oas.webservice.impl.test;

import java.util.List;
import java.util.Map;

import org.junit.Assert;
import org.junit.Test;
import org.openrdf.model.URI;
import org.restlet.data.CharacterSet;
import org.restlet.data.Language;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.representation.AppendableRepresentation;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;

import com.github.ansell.oas.objects.Ontology;
import com.github.ansell.oas.test.TestUtils;
import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.oas.utils.SesameUtils;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class CreateOntologyResourceImplTest extends AbstractResourceImplTest
{
    /**
     * Test whether Ontologies can be stored and retrieved using the application with both an
     * RDF/XML request and response. This test uses the special upload method with HTTP POST to
     * create the resource.
     * 
     * Specifically tests with ontologyUri, ontologyVersionUri, and baseUri but not
     * deleteOldVersions.
     */
    @Test
    public void testOntologyCreationRdfXml2RdfXml() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_UPLOAD_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_UPLOAD_PATH)));
        
        final String ontologyUri = "http://purl.org/obo/owl/test";
        final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        final String baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        // final Representation results = creationClientResource.post(input,
        // MediaType.APPLICATION_RDF_XML);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
        
        final Map<URI, List<Ontology>> createdOntologies =
                SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, createdOntologies.size());
        Assert.assertEquals(1, createdOntologies.keySet().size());
        Assert.assertEquals(1, createdOntologies.values().size());
        
        for(final URI nextUri : createdOntologies.keySet())
        {
            final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
            
            Assert.assertEquals(1, nextOntologies.size());
        }
    }
    
    /**
     * Test whether Ontologies can be stored and retrieved using the application with both an
     * RDF/XML request and response.
     * 
     * Specifically tests with ontologyUri, ontologyVersionUri, and baseUri with deleteOldVersions
     * set to false.
     */
    @Test
    public void testOntologyCreationRdfXml2RdfXmlDeleteOldVersionsFalse()
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_UPLOAD_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_UPLOAD_PATH)));
        
        final String ontologyUri = "http://purl.org/obo/owl/test";
        final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        final String baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("deleteOldVersions", "false");
        creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
    }
    
    /**
     * Test whether Ontologies can be stored and retrieved using the application with both an
     * RDF/XML request and response.
     * 
     * Specifically tests with ontologyUri, ontologyVersionUri, and baseUri but not
     * deleteOldVersions.
     * 
     * This test uses HTTP PUT, not HTTP POST which is the default for CreateOntologyResource.
     */
    @Test
    public void testOntologyCreationRdfXml2RdfXmlHttpPut() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_UPLOAD_PATH,
                        OasProps.DEF_ONTOLOGY_MANAGER_UPLOAD_PATH)));
        
        final String ontologyUri = "http://purl.org/obo/owl/test";
        final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
        final String baseUri = "http://purl.org/obo/owl/test#";
        
        creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
        creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
        creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
        
        // Send the draft annotation with a blank node as the base,
        // and the response will include triples with the assigned URI
        final String rdfDocument =
                TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
        
        final Representation input =
                new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
        
        final Map<URI, List<Ontology>> createdOntologies =
                SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, createdOntologies.size());
        Assert.assertEquals(1, createdOntologies.keySet().size());
        Assert.assertEquals(1, createdOntologies.values().size());
        
        for(final URI nextUri : createdOntologies.keySet())
        {
            final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
            
            Assert.assertEquals(1, nextOntologies.size());
        }
        
    }
}
