/**
 * 
 */
package com.github.ansell.oas.webservice.impl.test;

import org.junit.Assert;
import org.junit.Test;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.ext.html.FormData;
import org.restlet.ext.html.FormDataSet;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;
import org.restlet.resource.ResourceException;

import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class CreateUserResourceImplTest extends AbstractResourceImplTest
{
    
    /**
     * Tests that no error occurs when trying to get the create user resource while authenticated
     * with the admin role.
     */
    @Test
    public void testCreateUserGetBasicAuthorised() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_CREATE_USER_PATH,
                        OasProps.DEF_CREATE_USER_PATH)));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
    }
    
    /**
     * Tests that no error occurs when trying to post a valid form with authentication with all
     * fields filled.
     */
    @Test
    public void testCreateUserPostValidAuthenticatedFull() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_CREATE_USER_PATH,
                        OasProps.DEF_CREATE_USER_PATH)));
        
        final FormDataSet form = new FormDataSet();
        form.setMultipart(true);
        form.getEntries().add(new FormData("username", "testNewUser"));
        form.getEntries().add(new FormData("password", "testNewPassword"));
        form.getEntries().add(new FormData("email", "test@email.com"));
        form.getEntries().add(new FormData("lastName", "Usersmith"));
        form.getEntries().add(new FormData("firstName", "Mister"));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, form,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
        
        final ClientResource fetchClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_GET_USER_PATH,
                        OasProps.DEF_GET_USER_PATH)));
        fetchClientResource.getReference().addQueryParameter("username", "testNewUser");
        
        final Representation fetchResults =
                RestletTestUtils.doTestAuthenticatedRequest(fetchClientResource, Method.GET, null, MediaType.TEXT_HTML,
                        Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String fetchResult = fetchResults.getText();
        this.assertFreemarker(fetchResult);
    }
    
    /**
     * Tests that no error occurs when trying to post a valid form with authentication with only
     * required fields filled.
     */
    @Test
    public void testCreateUserPostValidAuthenticatedMinimal() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_CREATE_USER_PATH,
                        OasProps.DEF_CREATE_USER_PATH)));
        
        final FormDataSet form = new FormDataSet();
        form.setMultipart(true);
        form.getEntries().add(new FormData("username", "testNewUser"));
        form.getEntries().add(new FormData("password", "testNewPassword"));
        form.getEntries().add(new FormData("email", "test@email.com"));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, form,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
    }
    
    /**
     * Tests that an error occurs when trying to get the create user resource while unauthenticated.
     */
    @Test
    public void testErrorCreateUserGetNotAuthenticated() throws Exception
    {
        try
        {
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_CREATE_USER_PATH,
                            OasProps.DEF_CREATE_USER_PATH)));
            
            creationClientResource.get();
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    /**
     * Tests that an error occurs when trying to get the create user resource while authenticated
     * but without the admin role.
     */
    @Test
    public void testErrorCreateUserGetNotAuthorised() throws Exception
    {
        try
        {
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_CREATE_USER_PATH,
                            OasProps.DEF_CREATE_USER_PATH)));
            
            RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null, MediaType.TEXT_HTML,
                    Status.CLIENT_ERROR_UNAUTHORIZED, this.testNoAdminPrivileges);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    /**
     * Tests that an error occurs when trying to create a user without submitting a form
     */
    @Test
    public void testErrorCreateUserNoContent() throws Exception
    {
        try
        {
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_CREATE_USER_PATH,
                            OasProps.DEF_CREATE_USER_PATH)));
            
            RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, null,
                    MediaType.MULTIPART_FORM_DATA, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, rex.getStatus());
        }
    }
    
}
