/**
 * 
 */
package com.github.ansell.oas.webservice.impl.test;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import org.junit.Assert;
import org.junit.Test;
import org.openrdf.model.URI;
import org.restlet.data.ChallengeScheme;
import org.restlet.data.CharacterSet;
import org.restlet.data.Language;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.representation.AppendableRepresentation;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;
import org.restlet.resource.ResourceException;

import com.github.ansell.oas.objects.Ontology;
import com.github.ansell.oas.test.TestUtils;
import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.oas.utils.SesameUtils;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class DeleteOntologyResourceImplTest extends AbstractResourceImplTest
{
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * This test uses the base ontology manager with HTTP PUT to create the ontology originally,
     * before switching to the delete only path to test the DeleteOntologyResourceImpl.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter using HTTP POST, not HTTP DELETE.
     * 
     */
    @Test
    public void testOntologyDeleteAllVersionsRdfXml2RdfXml()
    {
        try
        {
            // Get the resource and dump it to the console
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            final String ontologyUri = "http://purl.org/obo/owl/test";
            final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
            final String baseUri = "http://purl.org/obo/owl/test#";
            
            creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
            creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String rdfDocument =
                    TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
            
            final Representation input =
                    new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            // final Representation results = creationClientResource.put(input,
            // MediaType.APPLICATION_RDF_XML);
            
            final Representation results =
                    RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            // Expecting HTTP 201 Created response
            // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
            // Expecting application/rdf+xml MIME type for response
            Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
            
            final Map<URI, List<Ontology>> createdOntologies =
                    SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
            
            Assert.assertEquals(1, createdOntologies.size());
            Assert.assertEquals(1, createdOntologies.keySet().size());
            Assert.assertEquals(1, createdOntologies.values().size());
            
            for(final URI nextUri : createdOntologies.keySet())
            {
                final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
                
                Assert.assertEquals(1, nextOntologies.size());
                
                final Ontology nextOntology = nextOntologies.get(0);
                
                Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
                Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
            }
            
            // Now that the ontology is uploaded and the response included a valid response we
            // assume it will be able to be deleted
            
            // Get the resource and dump it to the console
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_DELETE_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_DELETE_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            deletionClientResource.getReference().addQueryParameter("deleteAllVersions", "true");
            
            deletionClientResource.setChallengeResponse(ChallengeScheme.HTTP_BASIC, "user", "password");
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
        }
        catch(final Exception ex)
        {
            Assert.fail("Found unexpected exception when creating resource exception.toString=" + ex.toString());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter using HTTP Delete, not HTTP POST.
     * 
     */
    @Test
    public void testOntologyDeleteAllVersionsRdfXml2RdfXmlHttpDelete()
    {
        try
        {
            // Get the resource and dump it to the console
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            final String ontologyUri = "http://purl.org/obo/owl/test";
            final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
            final String baseUri = "http://purl.org/obo/owl/test#";
            
            creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
            creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String rdfDocument =
                    TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
            
            final Representation input =
                    new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            // final Representation results = creationClientResource.put(input,
            // MediaType.APPLICATION_RDF_XML);
            
            final Representation results =
                    RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            
            // Expecting HTTP 201 Created response
            // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
            // Expecting application/rdf+xml MIME type for response
            Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
            
            final Map<URI, List<Ontology>> createdOntologies =
                    SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
            
            Assert.assertEquals(1, createdOntologies.size());
            Assert.assertEquals(1, createdOntologies.keySet().size());
            Assert.assertEquals(1, createdOntologies.values().size());
            
            for(final URI nextUri : createdOntologies.keySet())
            {
                final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
                
                Assert.assertEquals(1, nextOntologies.size());
                
                final Ontology nextOntology = nextOntologies.get(0);
                
                Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
                Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
            }
            
            // Now that the ontology is uploaded and the response included a valid response we
            // assume it will be able to be deleted
            
            // Get the resource and dump it to the console
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_DELETE_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_DELETE_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            deletionClientResource.getReference().addQueryParameter("deleteAllVersions", "true");
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.DELETE, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
        }
        catch(final Exception ex)
        {
            Assert.fail("Found unexpected exception when creating resource exception.toString=" + ex.toString());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteNoUploadFalse()
    {
        try
        {
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_DELETE_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_DELETE_PATH)));
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteNoUploadWithDeleteAllFalse()
    {
        try
        {
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_DELETE_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_DELETE_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", "http://purl.org/obo/owl/test");
            deletionClientResource.getReference().addQueryParameter("deleteAllVersions", "true");
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteNoUploadWithOntologyUriFalse()
    {
        try
        {
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_DELETE_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_DELETE_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", "http://purl.org/obo/owl/test");
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteNoUploadWithVersionUriFalse()
    {
        try
        {
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_DELETE_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_DELETE_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", "http://purl.org/obo/owl/test");
            deletionClientResource.getReference().addQueryParameter("ontologyVersionUri",
                    "http://does.not.exist.example.org/test/delete/ontology");
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteNoURIAfterUploadRdfXml2RdfXmlFalse()
    {
        try
        {
            // Get the resource and dump it to the console
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            final String ontologyUri = "http://purl.org/obo/owl/test";
            final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
            final String baseUri = "http://purl.org/obo/owl/test#";
            
            creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
            creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String rdfDocument =
                    TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
            
            final Representation input =
                    new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            // final Representation results = creationClientResource.put(input,
            // MediaType.APPLICATION_RDF_XML);
            
            final Representation results =
                    RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            // Expecting HTTP 201 Created response
            // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
            // Expecting application/rdf+xml MIME type for response
            Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
            
            final Map<URI, List<Ontology>> createdOntologies =
                    SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
            
            Assert.assertEquals(1, createdOntologies.size());
            Assert.assertEquals(1, createdOntologies.keySet().size());
            Assert.assertEquals(1, createdOntologies.values().size());
            
            for(final URI nextUri : createdOntologies.keySet())
            {
                final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
                
                Assert.assertEquals(1, nextOntologies.size());
                
                final Ontology nextOntology = nextOntologies.get(0);
                
                Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
                Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
            }
            
            // Now that the ontology is uploaded and the response included a valid response we
            // assume it will be able to be deleted
            
            // Get the resource and dump it to the console
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_DELETE_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_DELETE_PATH)));
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
        catch(final IOException ioe)
        {
            Assert.fail("Found unexpected IOException: " + ioe.getMessage());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteNoVersionRdfXml2RdfXmlFalse()
    {
        try
        {
            // Get the resource and dump it to the console
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            final String ontologyUri = "http://purl.org/obo/owl/test";
            final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
            final String baseUri = "http://purl.org/obo/owl/test#";
            
            creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
            creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String rdfDocument =
                    TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
            
            final Representation input =
                    new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            // final Representation results = creationClientResource.put(input,
            // MediaType.APPLICATION_RDF_XML);
            
            final Representation results =
                    RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            // Expecting HTTP 201 Created response
            // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
            // Expecting application/rdf+xml MIME type for response
            Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
            
            final Map<URI, List<Ontology>> createdOntologies =
                    SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
            
            Assert.assertEquals(1, createdOntologies.size());
            Assert.assertEquals(1, createdOntologies.keySet().size());
            Assert.assertEquals(1, createdOntologies.values().size());
            
            for(final URI nextUri : createdOntologies.keySet())
            {
                final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
                
                Assert.assertEquals(1, nextOntologies.size());
                
                final Ontology nextOntology = nextOntologies.get(0);
                
                Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
                Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
            }
            
            // Now that the ontology is uploaded and the response included a valid response we
            // assume it will be able to be deleted
            
            // Get the resource and dump it to the console
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_DELETE_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_DELETE_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to call the delete service with just an ontology URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
        catch(final IOException ioe)
        {
            Assert.fail("Found unexpected IOException: " + ioe.getMessage());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteSingleVersionsRdfXml2RdfXmlFalse()
    {
        try
        {
            // Get the resource and dump it to the console
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            final String ontologyUri = "http://purl.org/obo/owl/test";
            final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
            final String baseUri = "http://purl.org/obo/owl/test#";
            
            creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
            creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String rdfDocument =
                    TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
            
            final Representation input =
                    new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            // final Representation results = creationClientResource.put(input,
            // MediaType.APPLICATION_RDF_XML);
            
            final Representation results =
                    RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            // Expecting HTTP 201 Created response
            // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
            // Expecting application/rdf+xml MIME type for response
            Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
            
            final Map<URI, List<Ontology>> createdOntologies =
                    SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
            
            Assert.assertEquals(1, createdOntologies.size());
            Assert.assertEquals(1, createdOntologies.keySet().size());
            Assert.assertEquals(1, createdOntologies.values().size());
            
            for(final URI nextUri : createdOntologies.keySet())
            {
                final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
                
                Assert.assertEquals(1, nextOntologies.size());
                
                final Ontology nextOntology = nextOntologies.get(0);
                
                Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
                Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
            }
            
            // Now that the ontology is uploaded and the response included a valid response we
            // assume it will be able to be deleted
            
            // Get the resource and dump it to the console
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_DELETE_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_DELETE_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            deletionClientResource.getReference().addQueryParameter("ontologyVersionUri",
                    "http://does.not.exist.example.org/test/delete/ontology");
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
            
            Assert.fail("Did not find expected exception while attempting to delete a non-existent ontology version URI");
        }
        catch(final ResourceException ex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, ex.getStatus());
        }
        catch(final IOException ioe)
        {
            Assert.fail("Found unexpected IOException: " + ioe.getMessage());
        }
    }
    
    /**
     * Test whether Ontologies can be deleted after creating the ontology using an RDF/XML file.
     * 
     * Specifically tests deletion with only the ontologyUri and no managedOntologyVersionUri
     * parameter.
     * 
     */
    @Test
    public void testOntologyDeleteSingleVersionsRdfXml2RdfXmlTrue()
    {
        try
        {
            // Get the resource and dump it to the console
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_PATH)));
            
            final String ontologyUri = "http://purl.org/obo/owl/test";
            final String ontologyVersionUri = "http://purl.org/obo/owl/test/version:1";
            final String baseUri = "http://purl.org/obo/owl/test#";
            
            creationClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            creationClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
            creationClientResource.getReference().addQueryParameter("baseUri", baseUri);
            
            // Send the draft annotation with a blank node as the base,
            // and the response will include triples with the assigned URI
            final String rdfDocument =
                    TestUtils.createTestOntologyDocument("application/rdf+xml", ontologyUri, ontologyVersionUri);
            
            final Representation input =
                    new AppendableRepresentation(rdfDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                            CharacterSet.UTF_8);
            
            // final Representation results = creationClientResource.put(input,
            // MediaType.APPLICATION_RDF_XML);
            
            final Representation results =
                    RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.PUT, input,
                            MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            // Expecting HTTP 201 Created response
            // Assert.assertEquals(201, creationClientResource.getResponse().getStatus().getCode());
            // Expecting application/rdf+xml MIME type for response
            Assert.assertEquals(MediaType.APPLICATION_RDF_XML.getName(), results.getMediaType().getName());
            
            final Map<URI, List<Ontology>> createdOntologies =
                    SesameUtils.parseOntologies(results.getStream(), baseUri, MediaType.APPLICATION_RDF_XML.getName());
            
            Assert.assertEquals(1, createdOntologies.size());
            Assert.assertEquals(1, createdOntologies.keySet().size());
            Assert.assertEquals(1, createdOntologies.values().size());
            
            for(final URI nextUri : createdOntologies.keySet())
            {
                final List<Ontology> nextOntologies = createdOntologies.get(nextUri);
                
                Assert.assertEquals(1, nextOntologies.size());
                
                final Ontology nextOntology = nextOntologies.get(0);
                
                Assert.assertEquals(ontologyUri, nextOntology.getOntologyUri().stringValue());
                Assert.assertEquals(ontologyVersionUri, nextOntology.getOntologyVersionUri().stringValue());
            }
            
            // Now that the ontology is uploaded and the response included a valid response we
            // assume it will be able to be deleted
            
            // Get the resource and dump it to the console
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ONTOLOGY_MANAGER_DELETE_PATH,
                            OasProps.DEF_ONTOLOGY_MANAGER_DELETE_PATH)));
            
            deletionClientResource.getReference().addQueryParameter("ontologyUri", ontologyUri);
            deletionClientResource.getReference().addQueryParameter("ontologyVersionUri", ontologyVersionUri);
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, null,
                    MediaType.TEXT_PLAIN, Status.SUCCESS_NO_CONTENT, this.testWithAdminPrivileges);
        }
        catch(final Exception ex)
        {
            Assert.fail("Found unexpected exception when creating resource exception.toString=" + ex.toString());
        }
    }
}
