/**
 * 
 */
package com.github.ansell.oas.webservice.impl.test;

import org.junit.Assert;
import org.junit.Test;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.ext.html.FormData;
import org.restlet.ext.html.FormDataSet;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;
import org.restlet.resource.ResourceException;

import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class DeleteUserResourceImplTest extends AbstractResourceImplTest
{
    
    /**
     * Tests that no error occurs when trying to post a valid form with authentication with only
     * required fields filled.
     */
    @Test
    public void testCreateAndDeleteUserPostValidAuthenticatedMinimal() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_CREATE_USER_PATH,
                        OasProps.DEF_CREATE_USER_PATH)));
        
        final FormDataSet form = new FormDataSet();
        form.setMultipart(true);
        form.getEntries().add(new FormData("username", "testNewUser"));
        form.getEntries().add(new FormData("password", "testNewPassword"));
        form.getEntries().add(new FormData("email", "test@email.com"));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, form,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
        
        final ClientResource deletionClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_DELETE_USER_PATH,
                        OasProps.DEF_DELETE_USER_PATH)));
        
        final FormDataSet deleteForm = new FormDataSet();
        deleteForm.setMultipart(false);
        deleteForm.setMediaType(MediaType.APPLICATION_WWW_FORM);
        deleteForm.add("username", "testNewUser");
        
        final Representation deleteResults =
                RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, deleteForm,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String deleteResult = deleteResults.getText();
        this.assertFreemarker(deleteResult);
    }
    
    /**
     * Tests that no error occurs when trying to get the create user resource while authenticated
     * with the admin role.
     */
    @Test
    public void testDeleteUserGetBasicAuthorised() throws Exception
    {
        final ClientResource deletionClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_DELETE_USER_PATH,
                        OasProps.DEF_DELETE_USER_PATH)));
        
        deletionClientResource.addQueryParameter("username", "anotherUser");
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.GET, null,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
    }
    
    /**
     * Tests that no error occurs when trying to get the create user resource while authenticated
     * with the admin role.
     */
    @Test
    public void testDeleteUserGetBasicAuthorisedNonExistent() throws Exception
    {
        final ClientResource deletionClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_DELETE_USER_PATH,
                        OasProps.DEF_DELETE_USER_PATH)));
        
        deletionClientResource.addQueryParameter("username", "nonExistentUser");
        
        try
        {
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.GET, null, MediaType.TEXT_HTML,
                    Status.SUCCESS_OK, this.testWithAdminPrivileges);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, rex.getStatus());
        }
    }
    
    /**
     * Tests that no error occurs when trying to get the create user resource while authenticated
     * with the admin role.
     */
    @Test
    public void testDeleteUserGetBasicNotAuthorisedNonExistent() throws Exception
    {
        final ClientResource deletionClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_DELETE_USER_PATH,
                        OasProps.DEF_DELETE_USER_PATH)));
        
        deletionClientResource.addQueryParameter("username", "nonExistentUser");
        
        try
        {
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.GET, null, MediaType.TEXT_HTML,
                    Status.SUCCESS_OK, this.testNoAdminPrivileges);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    /**
     * Tests that no error occurs when trying to post a valid form with authentication with all
     * fields filled.
     */
    @Test
    public void testDeleteUserPostValidAuthenticatedFull() throws Exception
    {
        final ClientResource deletionClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_DELETE_USER_PATH,
                        OasProps.DEF_DELETE_USER_PATH)));
        
        final FormDataSet form = new FormDataSet();
        form.setMultipart(false);
        form.setMediaType(MediaType.APPLICATION_WWW_FORM);
        form.add("username", "anotherUser");
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, form,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
    }
    
    /**
     * Tests that an error occurs when trying to get the create user resource while unauthenticated.
     */
    @Test
    public void testErrorDeleteUserGetNotAuthenticated() throws Exception
    {
        try
        {
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_DELETE_USER_PATH,
                            OasProps.DEF_DELETE_USER_PATH)));
            
            final FormDataSet form = new FormDataSet();
            form.setMultipart(false);
            form.setMediaType(MediaType.APPLICATION_WWW_FORM);
            form.add("username", "anotherUser");
            
            deletionClientResource.post(form);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    /**
     * Tests that an error occurs when trying to create a user without submitting a form
     */
    @Test
    public void testErrorDeleteUserNoContent() throws Exception
    {
        try
        {
            final ClientResource deletionClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_DELETE_USER_PATH,
                            OasProps.DEF_DELETE_USER_PATH)));
            
            RestletTestUtils.doTestAuthenticatedRequest(deletionClientResource, Method.POST, null,
                    MediaType.MULTIPART_FORM_DATA, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, rex.getStatus());
        }
    }
    
}
