/**
 * 
 */
package com.github.ansell.oas.webservice.impl.test;

import org.junit.Assert;
import org.junit.Test;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;
import org.restlet.resource.ResourceException;

import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class EditUserRolesResourceImplTest extends AbstractResourceImplTest
{
    /**
     * Tests that no error occurs when trying to get the edit user roles resource while
     * authenticated with the admin role.
     */
    @Test
    public void testEditUserRolesGetValidAuthorisedOther() throws Exception
    {
        this.createTestUser();
        
        final ClientResource editUserRolesClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                        OasProps.DEF_EDIT_USER_ROLES_PATH)));
        
        editUserRolesClientResource.addQueryParameter("username", "testNewUser");
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(editUserRolesClientResource, Method.GET, null,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String resultsText = results.getText();
        
        this.log.info("resultsText={}", resultsText);
        
        this.assertFreemarker(resultsText);
        Assert.assertTrue(resultsText.contains("OAS-testwebsite test edit role template"));
        Assert.assertTrue(resultsText.contains("Editing roles for "));
        Assert.assertTrue(resultsText.contains("Edit roles"));
        Assert.assertTrue(resultsText.contains("<option value=\"Administrator\">Administrator</option>"));
        // None of the existing assignable roles are assigned to the test user
        Assert.assertFalse(resultsText.contains("selected=\"selected\""));
    }
    
    /**
     * Tests that no error occurs when trying to get the edit user roles resource while
     * authenticated with the admin role.
     */
    @Test
    public void testEditUserRolesGetValidAuthorisedSelfExplicit() throws Exception
    {
        this.createTestUser();
        
        final ClientResource editUserRolesClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                        OasProps.DEF_EDIT_USER_ROLES_PATH)));
        
        editUserRolesClientResource.addQueryParameter("username", RestletTestUtils.TEST_ADMIN_USERNAME);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(editUserRolesClientResource, Method.GET, null,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String resultsText = results.getText();
        
        this.log.info("resultsText={}", resultsText);
        
        this.assertFreemarker(resultsText);
        Assert.assertTrue(resultsText.contains("OAS-testwebsite test edit role template"));
        Assert.assertTrue(resultsText.contains("Editing roles for "));
        Assert.assertTrue(resultsText.contains("Edit roles"));
        Assert.assertTrue(resultsText
                .contains("<option value=\"Administrator\" selected=\"selected\">Administrator</option>"));
        // Should have a role selected
        Assert.assertTrue(resultsText.contains("selected=\"selected\""));
    }
    
    /**
     * Tests that no error occurs when trying to get the edit user roles resource while
     * authenticated with the admin role.
     */
    @Test
    public void testEditUserRolesGetValidAuthorisedSelfImplicit() throws Exception
    {
        this.createTestUser();
        
        final ClientResource editUserRolesClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                        OasProps.DEF_EDIT_USER_ROLES_PATH)));
        
        // No username sent to test implicit support for editing roles for the current user who in
        // this case is an administrator
        // editUserRolesClientResource.addQueryParameter("username", "testNewUser");
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(editUserRolesClientResource, Method.GET, null,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String resultsText = results.getText();
        
        this.log.info("resultsText={}", resultsText);
        
        this.assertFreemarker(resultsText);
        Assert.assertTrue(resultsText.contains("OAS-testwebsite test edit role template"));
        Assert.assertTrue(resultsText.contains("Editing roles for "));
        Assert.assertTrue(resultsText.contains("Edit roles"));
        Assert.assertTrue(resultsText
                .contains("<option value=\"Administrator\" selected=\"selected\">Administrator</option>"));
        // Should have a role selected
        Assert.assertTrue(resultsText.contains("selected=\"selected\""));
    }
    
    /**
     * Tests that no error occurs when trying to get the edit user roles resource while
     * authenticated with the admin role.
     */
    @Test
    public void testEditUserRolesPostMultipleAuthorisedFail() throws Exception
    {
        this.createTestUser();
        
        final ClientResource editUserRolesClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                        OasProps.DEF_EDIT_USER_ROLES_PATH)));
        
        editUserRolesClientResource.addQueryParameter("username", "testNewUser");
        // attach two different roles, one of which is invalid
        editUserRolesClientResource.addQueryParameter("activeRoles", "Test Non-ExistentRole");
        editUserRolesClientResource.addQueryParameter("activeRoles", "Administrator");
        
        try
        {
            RestletTestUtils.doTestAuthenticatedRequest(editUserRolesClientResource, Method.POST, null,
                    MediaType.APPLICATION_WWW_FORM, Status.SUCCESS_OK, this.testWithAdminPrivileges);
            Assert.fail("Did not find expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, rex.getStatus());
        }
        
    }
    
    /**
     * Tests that no error occurs when trying to get the edit user roles resource while
     * authenticated with the admin role.
     */
    @Test
    public void testEditUserRolesPostValidAuthorised() throws Exception
    {
        this.createTestUser();
        
        final ClientResource editUserRolesClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                        OasProps.DEF_EDIT_USER_ROLES_PATH)));
        
        editUserRolesClientResource.addQueryParameter("username", "testNewUser");
        editUserRolesClientResource.addQueryParameter("activeRoles", "Administrator");
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(editUserRolesClientResource, Method.POST, null,
                        MediaType.APPLICATION_WWW_FORM, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
    }
    
    /**
     * Tests that no error occurs when trying to get the edit user roles resource while
     * authenticated with the admin role.
     */
    @Test
    public void testEditUserRolesPostValidAuthorisedRemovingOnlyAdmin() throws Exception
    {
        this.createTestUser();
        
        final ClientResource editUserRolesClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                        OasProps.DEF_EDIT_USER_ROLES_PATH)));
        
        editUserRolesClientResource.addQueryParameter("username", RestletTestUtils.TEST_ADMIN_USERNAME);
        editUserRolesClientResource.addQueryParameter("activeRoles", "");
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(editUserRolesClientResource, Method.POST, null,
                        MediaType.APPLICATION_WWW_FORM, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
        
        // Check that the administrator role still exists
        Assert.assertTrue(result.contains("Administrator,"));
        
        System.out.println(result);
    }
    
    /**
     * Tests that no error occurs when trying to get the edit user roles resource while
     * authenticated with the admin role.
     */
    @Test
    public void testEditUserRolesPostValidAuthorisedRemovingRedundantAdmin() throws Exception
    {
        this.createTestUser();
        
        final ClientResource editUserRolesClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                        OasProps.DEF_EDIT_USER_ROLES_PATH)));
        
        editUserRolesClientResource.addQueryParameter("username", RestletTestUtils.TEST_USERNAME);
        editUserRolesClientResource.addQueryParameter("activeRoles", "Administrator");
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(editUserRolesClientResource, Method.POST, null,
                        MediaType.APPLICATION_WWW_FORM, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
        
        // Check that the administrator role exists on the new administrators page
        Assert.assertTrue(result.contains("Administrator,"));
        
        // System.out.println(result);
        
        final ClientResource editUserRolesClientResource2 =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                        OasProps.DEF_EDIT_USER_ROLES_PATH)));
        
        editUserRolesClientResource2.addQueryParameter("username", RestletTestUtils.TEST_ADMIN_USERNAME);
        // Use the first admin user to remove their admin privileges
        editUserRolesClientResource2.addQueryParameter("activeRoles", "");
        
        final Representation results2 =
                RestletTestUtils.doTestAuthenticatedRequest(editUserRolesClientResource2, Method.POST, null,
                        MediaType.APPLICATION_WWW_FORM, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result2 = results2.getText();
        this.assertFreemarker(result2);
        
        // Check that the administrator role exists on the new administrators page
        Assert.assertFalse(result2.contains("Administrator,"));
        
        // System.out.println(result2);
        
        // Double check that they cannot now perform admin operations, such as getting back their
        // admin privileges since their is another admin
        final ClientResource editUserRolesClientResource3 =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                        OasProps.DEF_EDIT_USER_ROLES_PATH)));
        
        editUserRolesClientResource3.addQueryParameter("username", RestletTestUtils.TEST_ADMIN_USERNAME);
        editUserRolesClientResource3.addQueryParameter("activeRoles", "Administrator");
        
        try
        {
            RestletTestUtils.doTestAuthenticatedRequest(editUserRolesClientResource3, Method.POST, null,
                    MediaType.APPLICATION_WWW_FORM, Status.SUCCESS_OK, this.testWithAdminPrivileges);
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
        
    }
    
    /**
     * Tests that an error occurs when trying to get the edit user roles resource while
     * unauthenticated.
     */
    @Test
    public void testErrorCreateUserGetNotAuthenticated() throws Exception
    {
        try
        {
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                            OasProps.DEF_EDIT_USER_ROLES_PATH)));
            
            creationClientResource.get();
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    /**
     * Tests that an error occurs when trying to get the edit user roles resource while
     * authenticated but without the admin role.
     */
    @Test
    public void testErrorCreateUserGetNotAuthorised() throws Exception
    {
        try
        {
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                            OasProps.DEF_EDIT_USER_ROLES_PATH)));
            
            RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null, MediaType.TEXT_HTML,
                    Status.CLIENT_ERROR_UNAUTHORIZED, this.testNoAdminPrivileges);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    /**
     * Tests that an error occurs when trying to edit user roles without a valid form.
     */
    @Test
    public void testErrorEditUserRolesNoContent() throws Exception
    {
        try
        {
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_EDIT_USER_ROLES_PATH,
                            OasProps.PROP_EDIT_USER_ROLES_PATH)));
            
            RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, null,
                    MediaType.MULTIPART_FORM_DATA, Status.SUCCESS_CREATED, this.testWithAdminPrivileges);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, rex.getStatus());
        }
    }
}
