/**
 * OAS Webservice test implementations
 */
package com.github.ansell.oas.webservice.impl.test;

import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.Map;

import org.junit.Assert;
import org.junit.Test;
import org.openrdf.model.URI;
import org.restlet.data.CharacterSet;
import org.restlet.data.Language;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.representation.AppendableRepresentation;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;

import com.github.ansell.oas.objects.Annotation;
import com.github.ansell.oas.test.TestUtils;
import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.oas.utils.SesameUtils;
import com.github.ansell.restletutils.RestletUtilMediaType;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * Unit test for AnnotationResourceImpl.
 */
public class SearchAnnotationResourceImplTest extends AbstractResourceImplTest
{
    /**
     * Test whether Annotations can be searched using a JSON result type
     */
    @Test
    public void testAnnotationSearchJson() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        final String taggedResourceUri = "http://example.org/podd/sample/1";
        final String authorUri = "http://example.org/peter";
        final String tagName = "leaf measurement";
        final String ontologyUri = "http://purl.org/obo/owl/PO#PO_0006016";
        final String taggedResourceType = "http://www.podd.org/poddModel#Measurement";
        
        final String rdfJsonDocument =
                TestUtils.createTestAnnotationDocument(RestletUtilMediaType.APPLICATION_RDF_JSON.getName(), null,
                        taggedResourceUri, taggedResourceType, ontologyUri, tagName, authorUri);
        
        final Representation input =
                new AppendableRepresentation(rdfJsonDocument, RestletUtilMediaType.APPLICATION_RDF_JSON,
                        Language.DEFAULT, CharacterSet.UTF_8);
        
        final Representation createRepresentation =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_JSON, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        
        // Get the resource and dump it to the console
        final ClientResource searchAnnotationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_SEARCH_ANNOTATION_PATH,
                        OasProps.DEF_SEARCH_ANNOTATION_PATH)));
        
        searchAnnotationClientResource.getReference().addQueryParameter("searchterm", "leaf");
        
        final Representation searchResults =
                searchAnnotationClientResource.get(RestletUtilMediaType.APPLICATION_RDF_JSON);
        
        final Map<URI, Annotation> createdAnnotations =
                SesameUtils.parseAnnotations(new InputStreamReader(searchResults.getStream(), StandardCharsets.UTF_8),
                        "", RestletUtilMediaType.APPLICATION_RDF_JSON.getName());
        
        Assert.assertEquals(1, createdAnnotations.size());
        
    }
    
    /**
     * Test whether Annotations can be searched using an RDF/XML result type.
     */
    @Test
    public void testAnnotationSearchRdfXml() throws Exception
    {
        // Get the resource and dump it to the console
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_ANNOTATION_CREATE_PATH,
                        OasProps.DEF_ANNOTATIONS_CREATE_PATH)));
        
        final String taggedResourceUri = "http://example.org/podd/sample/1";
        final String authorUri = "http://example.org/peter";
        final String tagName = "leaf measurement";
        final String ontologyUri = "http://purl.org/obo/owl/PO#PO_0006016";
        final String taggedResourceType = "http://www.podd.org/poddModel#Measurement";
        
        final String rdfJsonDocument =
                TestUtils.createTestAnnotationDocument("application/rdf+xml", null, taggedResourceUri,
                        taggedResourceType, ontologyUri, tagName, authorUri);
        
        final Representation input =
                new AppendableRepresentation(rdfJsonDocument, MediaType.APPLICATION_RDF_XML, Language.DEFAULT,
                        CharacterSet.UTF_8);
        
        final Representation createRepresentation =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, input,
                        MediaType.APPLICATION_RDF_XML, Status.SUCCESS_CREATED, this.testNoAdminPrivileges);
        
        // Get the resource and dump it to the console
        final ClientResource searchAnnotationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_SEARCH_ANNOTATION_PATH,
                        OasProps.DEF_SEARCH_ANNOTATION_PATH)));
        
        searchAnnotationClientResource.getReference().addQueryParameter("searchterm", "leaf");
        
        final Representation searchResults = searchAnnotationClientResource.get(MediaType.APPLICATION_RDF_XML);
        
        final Map<URI, Annotation> createdAnnotations =
                SesameUtils.parseAnnotations(new InputStreamReader(searchResults.getStream(), StandardCharsets.UTF_8),
                        "", MediaType.APPLICATION_RDF_XML.getName());
        
        Assert.assertEquals(1, createdAnnotations.size());
        
    }
    
}
