/**
 * 
 */
package com.github.ansell.oas.webservice.impl.test;

import org.junit.Assert;
import org.junit.Test;
import org.restlet.data.MediaType;
import org.restlet.data.Method;
import org.restlet.data.Status;
import org.restlet.ext.html.FormData;
import org.restlet.ext.html.FormDataSet;
import org.restlet.representation.Representation;
import org.restlet.resource.ClientResource;
import org.restlet.resource.ResourceException;

import com.github.ansell.oas.utils.OasProps;
import com.github.ansell.restletutils.test.RestletTestUtils;

/**
 * @author Peter Ansell p_ansell@yahoo.com
 * 
 */
public class UpdateUserResourceImplTest extends AbstractResourceImplTest
{
    /**
     * Tests that an error occurs when trying to get the update user resource while authenticated
     * but without the admin role and trying to update another users information.
     */
    @Test
    public void testErrorUpdateUserPostForbidden() throws Exception
    {
        try
        {
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_UPDATE_USER_PATH,
                            OasProps.PROP_UPDATE_USER_PATH)));
            
            final FormDataSet form = new FormDataSet();
            form.setMultipart(false);
            form.setMediaType(MediaType.APPLICATION_WWW_FORM);
            form.getEntries().add(new FormData("username", "testAdminUser"));
            form.getEntries().add(new FormData("password", "testNewPassword"));
            form.getEntries().add(new FormData("email", "test@email.com"));
            
            RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, form, MediaType.TEXT_HTML,
                    Status.CLIENT_ERROR_UNAUTHORIZED, this.testNoAdminPrivileges);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    /**
     * Tests that an error occurs when trying to create a user without submitting a form
     */
    @Test
    public void testErrorUpdateUserPostNoContent() throws Exception
    {
        try
        {
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_UPDATE_USER_PATH,
                            OasProps.PROP_UPDATE_USER_PATH)));
            
            final FormDataSet form = new FormDataSet();
            form.setMultipart(false);
            form.setMediaType(MediaType.APPLICATION_WWW_FORM);
            creationClientResource.post(form);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, rex.getStatus());
        }
    }
    
    /**
     * Tests that an error occurs when trying to get the create user resource while unauthenticated.
     */
    @Test
    public void testErrorUpdateUserPostNotAuthenticated() throws Exception
    {
        try
        {
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_UPDATE_USER_PATH,
                            OasProps.PROP_UPDATE_USER_PATH)));
            
            final FormDataSet form = new FormDataSet();
            form.setMultipart(false);
            form.setMediaType(MediaType.APPLICATION_WWW_FORM);
            form.getEntries().add(new FormData("username", "testNewUser"));
            form.getEntries().add(new FormData("password", "testNewPassword"));
            form.getEntries().add(new FormData("email", "test@email.com"));
            
            creationClientResource.post(form);
            
            Assert.fail("Did not receive expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_UNAUTHORIZED, rex.getStatus());
        }
    }
    
    /**
     * Tests that no error occurs when trying to get the get update user resource while
     * authenticated with the admin role.
     */
    @Test
    public void testGetUpdateUserBasicAuthorised() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_UPDATE_USER_PATH,
                        OasProps.PROP_UPDATE_USER_PATH)) + "?username=" + RestletTestUtils.TEST_ADMIN_USERNAME);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
    }
    
    /**
     * Tests that no error occurs when trying to get the get user resource while authenticated with
     * the admin role.
     */
    @Test
    public void testGetUserBasicAuthorisedNonAdmin() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_UPDATE_USER_PATH,
                        OasProps.PROP_UPDATE_USER_PATH)) + "?username=" + RestletTestUtils.TEST_USERNAME);
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.GET, null,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testNoAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
    }
    
    /**
     * Tests that no error occurs when trying to post a valid form with authentication with only
     * required fields filled.
     */
    @Test
    public void testUpdateUserPostInvalidUsernameAuthenticatedMinimal() throws Exception
    {
        try
        {
            final ClientResource creationClientResource =
                    new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_UPDATE_USER_PATH,
                            OasProps.DEF_UPDATE_USER_PATH)));
            
            final FormDataSet form = new FormDataSet();
            form.setMultipart(false);
            form.setMediaType(MediaType.APPLICATION_WWW_FORM);
            form.getEntries().add(new FormData("username", "fakeUserName"));
            form.getEntries().add(new FormData("password", "testNewPassword"));
            form.getEntries().add(new FormData("email", "test@email.com"));
            
            RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, form, MediaType.TEXT_HTML,
                    Status.SUCCESS_OK, this.testWithAdminPrivileges);
            
            Assert.fail("Did not receive the expected exception");
        }
        catch(final ResourceException rex)
        {
            Assert.assertEquals(Status.CLIENT_ERROR_BAD_REQUEST, rex.getStatus());
        }
    }
    
    /**
     * Tests that no error occurs when trying to post a valid form with authentication with all
     * fields filled.
     */
    @Test
    public void testUpdateUserPostValidAuthenticatedFull() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_UPDATE_USER_PATH,
                        OasProps.DEF_UPDATE_USER_PATH)));
        
        final FormDataSet form = new FormDataSet();
        form.setMultipart(false);
        form.setMediaType(MediaType.APPLICATION_WWW_FORM);
        form.getEntries().add(new FormData("username", "anotherUser"));
        form.getEntries().add(new FormData("password", "testNewPassword"));
        form.getEntries().add(new FormData("email", "test@email.com"));
        form.getEntries().add(new FormData("lastName", "Usersmith"));
        form.getEntries().add(new FormData("firstName", "Mister"));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, form,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
        // FIXME: Check that the details for the user were updated
    }
    
    /**
     * Tests that no error occurs when trying to post a valid form with authentication with only
     * required fields filled.
     */
    @Test
    public void testUpdateUserPostValidAuthenticatedMinimal() throws Exception
    {
        final ClientResource creationClientResource =
                new ClientResource(this.getUrl(this.propertyUtil.get(OasProps.PROP_UPDATE_USER_PATH,
                        OasProps.DEF_UPDATE_USER_PATH)));
        
        final FormDataSet form = new FormDataSet();
        form.setMultipart(false);
        form.setMediaType(MediaType.APPLICATION_WWW_FORM);
        form.getEntries().add(new FormData("username", "anotherUser"));
        form.getEntries().add(new FormData("password", "testNewPassword"));
        form.getEntries().add(new FormData("email", "test@email.com"));
        
        final Representation results =
                RestletTestUtils.doTestAuthenticatedRequest(creationClientResource, Method.POST, form,
                        MediaType.TEXT_HTML, Status.SUCCESS_OK, this.testWithAdminPrivileges);
        
        final String result = results.getText();
        this.assertFreemarker(result);
    }
    
}
